@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: control.c,v 143.1 1996/09/16 09:08:30 nau Exp $";
+static	char	*rcsid = "$Id: control.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 /* control panel
  */
@@ -57,7 +57,7 @@ static	char	*rcsid = "$Id: control.c,v 143.1 1996/09/16 09:08:30 nau Exp $";
 /* ---------------------------------------------------------------------------
  * some local defines
  */
-#define	BUTTONS_PER_ROW		3	/* number of mode buttons per row */
+#define	BUTTONS_PER_ROW		2	/* number of mode buttons per row */
 
 /* ---------------------------------------------------------------------------
  * some local types
@@ -105,23 +105,25 @@ static	ModeButtonType	ModeButtons[] = {
 		ViaModeIcon_width, ViaModeIcon_height, NULL },
 	{ "lineIcon", "line", LINE_MODE, LineModeIcon_bits,
 		LineModeIcon_width, LineModeIcon_height, NULL },
-	{ "rectangleIcon", "rectangle", RECTANGLE_MODE, RectModeIcon_bits,
-		RectModeIcon_width, RectModeIcon_height, NULL },
+	{ "arcIcon", "arc", ARC_MODE, ArcModeIcon_bits,
+		ArcModeIcon_width, ArcModeIcon_height, NULL},
 	{ "textIcon", "text", TEXT_MODE, TextModeIcon_bits,
 		TextModeIcon_width, TextModeIcon_height, NULL },
+	{ "rectangleIcon", "rectangle", RECTANGLE_MODE, RectModeIcon_bits,
+		RectModeIcon_width, RectModeIcon_height, NULL },
 	{ "polygonIcon", "polygon", POLYGON_MODE, PolygonModeIcon_bits,
 		PolygonModeIcon_width, PolygonModeIcon_height, NULL },
 	{ "bufferIcon", "buffer", PASTEBUFFER_MODE, BufferModeIcon_bits,
 		BufferModeIcon_width, BufferModeIcon_height, NULL },
 	{ "removeIcon", "remove", REMOVE_MODE, RemoveModeIcon_bits,
 		RemoveModeIcon_width, RemoveModeIcon_height, NULL },
-	{ "mirrorIcon", "mirror", MIRROR_MODE, MirrorModeIcon_bits,
-		MirrorModeIcon_width, MirrorModeIcon_height, NULL },
 	{ "rotateIcon", "rotate", ROTATE_MODE, RotateModeIcon_bits,
 		RotateModeIcon_width, RotateModeIcon_height, NULL },
 	{ "insertPointIcon", "insertPoint", INSERTPOINT_MODE,
 		InsertPointModeIcon_bits, InsertPointModeIcon_width,
-		InsertPointModeIcon_height, NULL }};
+		InsertPointModeIcon_height, NULL },
+	{ "thermalIcon", "thermal", THERMAL_MODE, ThermalModeIcon_bits,
+		ThermalModeIcon_width, ThermalModeIcon_height, NULL}};
 
 /* ---------------------------------------------------------------------------
  * adds a label without border, surrounded by additional space
@@ -171,7 +173,7 @@ static void AddDrawingLayerSelection(Widget Parent)
 		/* init routine exits on NULL pointer */
 	DrawingLayerMenuEntries[i].Name = NULL;
 	DrawingLayerMenuEntries[i].Label = NULL;
-	label = AddLabel("current", Parent, NULL, NULL);
+	label = AddLabel("active", Parent, NULL, NULL);
 	InitMenuButton(Parent, &DrawingLayerMenuButton, label, NULL);
 	XtVaGetValues(DrawingLayerMenuButton.W,
 		XtNbackground, &background,
@@ -202,13 +204,13 @@ static void AddOnOffSelection(Widget Parent)
 		{
 			case MAX_LAYER:			/* settings for elements */
 				color = PCB->ElementColor;
-				text = "pkg. outline";
+				text = "silk";
 				strcpy(name[i], "elements");
 				break;
 
 			case MAX_LAYER+1:		/* settings for pins */
 				color = PCB->PinColor;
-				text = "pkg. pins";
+				text = "pins/pads";
 				strcpy(name[i], "pins");
 				break;
 
@@ -220,7 +222,7 @@ static void AddOnOffSelection(Widget Parent)
 
 			case MAX_LAYER+3:		/* settings for invisible objects */
 				color = PCB->InvisibleObjectsColor;
-				text = "invis. obj.";
+				text = "far side";
 				strcpy(name[i], "invisible");
 				break;
 
@@ -325,7 +327,7 @@ int ChangeGroupVisibility(Cardinal Layer, Boolean On, Boolean ChangeStackOrder)
 			int	layer = PCB->LayerGroups.Entries[group][--i];
 
 				/* dont count the passed member of the group */
-			if (layer != Layer)
+			if (layer != Layer && layer < MAX_LAYER)
 			{
 				PCB->Data->Layer[layer].On = On;
 	
