@@ -24,12 +24,13 @@
  *
  */
 
-static	char	*rcsid = "$Id: crosshair.c,v 143.1 1996/09/16 09:08:32 nau Exp $";
+static	char	*rcsid = "$Id: crosshair.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 
 /* crosshair stuff
  */
 
 #include <memory.h>
+#include <math.h>
 
 #include "global.h"
 
@@ -37,6 +38,7 @@ static	char	*rcsid = "$Id: crosshair.c,v 143.1 1996/09/16 09:08:32 nau Exp $";
 #include "data.h"
 #include "draw.h"
 #include "error.h"
+#include "misc.h"
 #include "mymem.h"
 
 #include <X11/Xaw/Simple.h>
@@ -45,9 +47,9 @@ static	char	*rcsid = "$Id: crosshair.c,v 143.1 1996/09/16 09:08:32 nau Exp $";
  * some local identifiers
  */
 static	Boolean		CrosshairStack[MAX_CROSSHAIRSTACK_DEPTH];
-static	int			CrosshairStackPosition = 0;
+static	int		CrosshairStackPosition = 0;
 static	XPoint		*Points = NULL;		/* data of tmp polygon */
-static	Cardinal	MaxPoints = 0;				/* number of points */
+static	Cardinal	MaxPoints = 0;		/* number of points */
 
 /* ---------------------------------------------------------------------------
  * some local prototypes
@@ -58,6 +60,8 @@ static	void	XORDrawElement(ElementTypePtr, Position, Position);
 static	void	XORDrawBuffer(BufferTypePtr);
 static	void	XORDrawInsertPointObject(void);
 static	void	XORDrawMoveOrCopyObject(void);
+static  void    XORDrawAttachedLine(Position, Position, Position, Position, Dimension);
+static	void	XORDrawAttachedArc(Dimension);
 static	void	DrawAttached(Boolean);
 static	void	FitCrosshairIntoGrid(Position, Position);
 
@@ -88,15 +92,119 @@ static void CreateTMPPolygon(PolygonTypePtr Polygon, Position DX, Position DY)
 
 /* ---------------------------------------------------------------------------
  * draws the crosshair 
- * don't transform MAX_COORD to screen coordinats, it is
- * already the maximum of screen- and pcb-coordinates
+ * don't transform MAX_COORD to screen coordinates, it is
+ * already the maximum of screen and pcb-coordinates
  */
 static void DrawCrosshair(void)
 {
 	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-		TO_SCREEN_X(Crosshair.X), 0, TO_SCREEN_X(Crosshair.X), MAX_COORD);
+		TO_SCREEN_X(Crosshair.X), 0,
+		TO_SCREEN_X(Crosshair.X), MAX_COORD);
+
 	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-		0, TO_SCREEN_Y(Crosshair.Y), MAX_COORD, TO_SCREEN_Y(Crosshair.Y));
+		0, TO_SCREEN_Y(Crosshair.Y), MAX_COORD,
+		TO_SCREEN_Y(Crosshair.Y));
+}
+
+/*-----------------------------------------------------------
+ * Draws the outline of an arc
+ */
+static void	XORDrawAttachedArc(Dimension thick)
+{
+	ArcType	arc;
+	BoxTypePtr bx;
+	Position wx, wy;
+	int sa, dir;
+	Dimension wid = thick/2;
+
+	wx = Crosshair.X - Crosshair.AttachedBox.Point1.X;
+	wy = Crosshair.Y - Crosshair.AttachedBox.Point1.Y;
+	if (wx == 0 && wy == 0)
+		return;
+	arc.X = Crosshair.AttachedBox.Point1.X;
+	arc.Y = Crosshair.AttachedBox.Point1.Y;
+	if (abs(wy) > abs(wx))
+	{
+		arc.X = Crosshair.AttachedBox.Point1.X + abs(wy)*SGN(wx);
+		sa = (wx >= 0) ? 0 : 180;
+#ifdef ARC45
+		if (abs(wy) >= 2*abs(wx))
+			dir = (SGN(wx) == SGN(wy)) ? 45 : -45;
+		else
+#endif
+			dir = (SGN(wx) == SGN(wy)) ? 90 : -90;
+		}
+	else
+	{
+		arc.Y = Crosshair.AttachedBox.Point1.Y + abs(wx)*SGN(wy);
+		sa = (wy >= 0) ? -90 : 90;
+#ifdef ARC45
+		if (abs(wx) >= 2*abs(wy))
+			dir = (SGN(wx) == SGN(wy)) ? -45 : 45;
+		else
+#endif
+			dir = (SGN(wx) == SGN(wy)) ? -90 : 90;
+		wy = wx;
+	}
+	wy = abs(wy);
+	arc.StartAngle = sa;
+	arc.Delta = dir;
+	arc.Width = arc.Height = wy;
+	bx = GetArcEnds(&arc);
+	arc.X -= wy;
+	arc.Y -= wy;
+	sa = (sa - 180) * 64;
+	dir *= 64;
+	XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(arc.X - wid), TO_SCREEN_Y(arc.Y - wid),
+		TO_SCREEN(2*wy + thick), TO_SCREEN(2*wy + thick), sa, dir);
+	if (TO_SCREEN(wid) && (2*wy - thick) > 0 && TO_SCREEN(2*wy - thick))
+	{
+		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(arc.X + wid), TO_SCREEN_Y(arc.Y + wid),
+			TO_SCREEN(2*wy - thick), TO_SCREEN(2*wy - thick), sa, dir);
+		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(bx->X1 - wid), TO_SCREEN_Y(bx->Y1 - wid),
+			TO_SCREEN(thick), TO_SCREEN(thick), sa, -11520*SGN(dir));
+		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(bx->X2 - wid), TO_SCREEN_Y(bx->Y2 - wid),
+			TO_SCREEN(thick), TO_SCREEN(thick), sa + dir, 11520*SGN(dir));
+	}
+}
+
+/*-----------------------------------------------------------
+ * Draws the outline of a line
+ */
+static void	XORDrawAttachedLine(Position x1, Position y1, Position x2,
+				Position y2, Dimension thick)
+{
+	Position dx,dy, ox, oy;
+	float h;
+	Dimension wid = thick/2;
+
+	dx = x2 - x1;
+	dy = y2 - y1;
+	h = wid / sqrt((float) (dx * dx + dy * dy));
+	ox = dy * h;
+	oy = - dx * h;
+	XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+		TO_SCREEN_X(x1 + ox), TO_SCREEN_Y(y1 + oy),
+		TO_SCREEN_X(x2 + ox), TO_SCREEN_Y(y2 + oy));
+	if (TO_SCREEN(abs(ox)) || TO_SCREEN(abs(oy)))
+	{
+		Position angle = atan(((float) dx)/dy) * 3666.9298888;
+		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(x1 - ox), TO_SCREEN_Y(y1 - oy),
+			TO_SCREEN_X(x2 - ox), TO_SCREEN_Y(y2 - oy));
+		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(x1 - wid), TO_SCREEN_Y(y1 - wid),
+			TO_SCREEN(thick), TO_SCREEN(thick), angle,
+			dy < 0 ? -11520 : 11520);
+		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+			TO_SCREEN_X(x2 - wid), TO_SCREEN_Y(y2 - wid),
+			TO_SCREEN(thick), TO_SCREEN(thick), angle,
+			dy < 0 ? 11520 : -11520);
+	}
 }
 
 /* ---------------------------------------------------------------------------
@@ -106,37 +214,39 @@ static void XORDrawElement(ElementTypePtr Element, Position DX, Position DY)
 {
 	ELEMENTLINE_LOOP(Element,
 		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(DX +line->Point1.X), TO_SCREEN_Y(DY +line->Point1.Y),
-			TO_SCREEN_X(DX +line->Point2.X), TO_SCREEN_Y(DY +line->Point2.Y));
-	);
+			TO_SCREEN_X(DX +line->Point1.X),
+			TO_SCREEN_Y(DY +line->Point1.Y),
+			TO_SCREEN_X(DX +line->Point2.X),
+			TO_SCREEN_Y(DY +line->Point2.Y)););
 
 		/* arc coordinates and angles have to be converted to X11 notation */
 	ARC_LOOP(Element,
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 			TO_SCREEN_X(DX +arc->X) -TO_SCREEN(arc->Width),
 			TO_SCREEN_Y(DY +arc->Y) -TO_SCREEN(arc->Height),
-			TO_SCREEN(2*arc->Width), TO_SCREEN(2*arc->Height),
+			TO_SCREEN(2*arc->Width),
+			TO_SCREEN(2*arc->Height),
 			(TO_SCREEN_ANGLE(arc->StartAngle) +180) *64,
-			TO_SCREEN_DELTA(arc->Delta) *64);
-	);
+			TO_SCREEN_DELTA(arc->Delta) *64););
 
 		/* pin coordinates and angles have to be converted to X11 notation */
 	PIN_LOOP(Element,
 		XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 			TO_SCREEN_X(DX +pin->X) -TO_SCREEN(pin->Thickness/2),
 			TO_SCREEN_Y(DY +pin->Y) -TO_SCREEN(pin->Thickness/2),
-			TO_SCREEN(pin->Thickness), TO_SCREEN(pin->Thickness),
-			0, 360*64);
-	);
+			TO_SCREEN(pin->Thickness),
+			TO_SCREEN(pin->Thickness),
+			0, 360*64););
 
 		/* pads */
 	PAD_LOOP(Element,
 		if ((TEST_FLAG(ONSOLDERFLAG, pad) != 0) == Settings.ShowSolderSide ||
 			PCB->InvisibleObjectsOn)
 			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(DX +pad->Point1.X), TO_SCREEN_Y(DY +pad->Point1.Y),
-				TO_SCREEN_X(DX +pad->Point2.X), TO_SCREEN_Y(DY +pad->Point2.Y));
-	);
+				TO_SCREEN_X(DX +pad->Point1.X),
+				TO_SCREEN_Y(DY +pad->Point1.Y),
+				TO_SCREEN_X(DX +pad->Point2.X),
+				TO_SCREEN_Y(DY +pad->Point2.Y)););
 
 }
 
@@ -160,15 +270,27 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 
 			LINE_LOOP(layer,
 				XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-					TO_SCREEN_X(x +line->Point1.X), TO_SCREEN_Y(y +line->Point1.Y),
-					TO_SCREEN_X(x +line->Point2.X), TO_SCREEN_Y(y +line->Point2.Y));
+					TO_SCREEN_X(x +line->Point1.X),
+					TO_SCREEN_Y(y +line->Point1.Y),
+					TO_SCREEN_X(x +line->Point2.X),
+					TO_SCREEN_Y(y +line->Point2.Y));
+			);
+			ARC_LOOP(layer,
+				XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+                			TO_SCREEN_X(x + arc->X) -TO_SCREEN(arc->Width),
+                			TO_SCREEN_Y(y + arc->Y) -TO_SCREEN(arc->Height),
+                			TO_SCREEN(2*arc->Width),
+                			TO_SCREEN(2*arc->Height),
+                			(TO_SCREEN_ANGLE(arc->StartAngle) -180) *64,
+                			TO_SCREEN_DELTA(arc->Delta) *64);
 			);
 			TEXT_LOOP(layer,
 				{
 					BoxTypePtr	box = &text->BoundingBox;
 
 					XDrawRectangle(Dpy, Output.OutputWindow, Crosshair.GC,
-						TO_SCREEN_X(x +box->X1), TO_SCREEN_Y(y +box->Y1),
+						TO_SCREEN_X(x +box->X1),
+						TO_SCREEN_Y(y +box->Y1),
 						TO_SCREEN(box->X2 -box->X1),
 						TO_SCREEN(box->Y2 -box->Y1));
 				}
@@ -186,7 +308,7 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
 		}
 
 		/* draw elements if visible */
-	if (PCB->PinOn || PCB->ElementOn)
+	if (PCB->PinOn && PCB->ElementOn)
 		ELEMENT_LOOP(Buffer->Data,
 			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
 			PCB->InvisibleObjectsOn)
@@ -209,43 +331,19 @@ static void XORDrawBuffer(BufferTypePtr Buffer)
  */
 static void XORDrawInsertPointObject(void)
 {
-		/* decicion is based onthe parent object */
-	switch(Crosshair.AttachedObject.Type)
-	{
-		case LINE_TYPE:
-		{
-			LineTypePtr	line = (LineTypePtr) Crosshair.AttachedObject.Ptr2;
+        LineTypePtr     line = (LineTypePtr) Crosshair.AttachedObject.Ptr2;
+        PointTypePtr    point = (PointTypePtr) Crosshair.AttachedObject.Ptr3;
 
-			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(Crosshair.X), TO_SCREEN_Y(Crosshair.Y),
-				TO_SCREEN_X(line->Point1.X), TO_SCREEN_Y(line->Point1.Y));
-			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(Crosshair.X), TO_SCREEN_Y(Crosshair.Y),
-				TO_SCREEN_X(line->Point2.X), TO_SCREEN_Y(line->Point2.Y));
-		}
-		break;
+        if (Crosshair.AttachedObject.Type != NO_TYPE)
+        {
+                        XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+                        TO_SCREEN_X(point->X), TO_SCREEN_Y(point->Y),
+                                TO_SCREEN_X(line->Point1.X), TO_SCREEN_Y(line->Point1.Y));
+                        XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+                        TO_SCREEN_X(point->X), TO_SCREEN_Y(point->Y),
+                                TO_SCREEN_X(line->Point2.X), TO_SCREEN_Y(line->Point2.Y));
+         }       
 
-		case POLYGON_TYPE:
-		{
-			PolygonTypePtr	polygon;
-			PointTypePtr	first,
-							second;
-			
-			polygon = (PolygonTypePtr) Crosshair.AttachedObject.Ptr2;
-			second = (PointTypePtr) Crosshair.AttachedObject.Ptr3;
-
-				/* take care about overflow */
-			first = (second == polygon->Points ?
-				&polygon->Points[polygon->PointN-1] : second -1);
-			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(Crosshair.X), TO_SCREEN_Y(Crosshair.Y),
-				TO_SCREEN_X(first->X), TO_SCREEN_Y(first->Y));
-			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(Crosshair.X), TO_SCREEN_Y(Crosshair.Y),
-				TO_SCREEN_X(second->X), TO_SCREEN_Y(second->Y));
-		}
-		break;
-	}
 }
 
 /* ---------------------------------------------------------------------------
@@ -254,9 +352,9 @@ static void XORDrawInsertPointObject(void)
 static void XORDrawMoveOrCopyObject(void)
 {
 	RubberbandTypePtr	ptr;
-	Cardinal			i;
-	Position			dx = Crosshair.X -Crosshair.AttachedObject.X,
-						dy = Crosshair.Y -Crosshair.AttachedObject.Y;
+	Cardinal		i;
+	Position		dx = Crosshair.X -Crosshair.AttachedObject.X,
+				dy = Crosshair.Y -Crosshair.AttachedObject.Y;
 
 	switch(Crosshair.AttachedObject.Type)
 	{
@@ -267,7 +365,8 @@ static void XORDrawMoveOrCopyObject(void)
 			XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
 				TO_SCREEN_X(via->X +dx -via->Thickness/2),
 				TO_SCREEN_Y(via->Y +dy -via->Thickness/2),
-				TO_SCREEN(via->Thickness), TO_SCREEN(via->Thickness),0,360*64);
+				TO_SCREEN(via->Thickness),
+				TO_SCREEN(via->Thickness),0,360*64);
 			break;
 		}
 
@@ -276,8 +375,24 @@ static void XORDrawMoveOrCopyObject(void)
 			LineTypePtr	line = (LineTypePtr) Crosshair.AttachedObject.Ptr2;
 
 			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(line->Point1.X +dx), TO_SCREEN_Y(line->Point1.Y +dy),
-				TO_SCREEN_X(line->Point2.X +dx), TO_SCREEN_Y(line->Point2.Y +dy));
+				TO_SCREEN_X(line->Point1.X +dx),
+				TO_SCREEN_Y(line->Point1.Y +dy),
+				TO_SCREEN_X(line->Point2.X +dx),
+				TO_SCREEN_Y(line->Point2.Y +dy));
+			break;
+		}
+
+		case ARC_TYPE:
+		{
+			ArcTypePtr	Arc = (ArcTypePtr) Crosshair.AttachedObject.Ptr2;
+
+			XDrawArc(Dpy, Output.OutputWindow, Crosshair.GC,
+                		TO_SCREEN_X(Arc->X +dx) -TO_SCREEN(Arc->Width),
+                		TO_SCREEN_Y(Arc->Y +dy) -TO_SCREEN(Arc->Height),
+                		TO_SCREEN(2*Arc->Width),
+                		TO_SCREEN(2*Arc->Height),
+                		(TO_SCREEN_ANGLE(Arc->StartAngle) -180) *64,
+                		TO_SCREEN_DELTA(Arc->Delta) *64);
 			break;
 		}
 
@@ -296,19 +411,24 @@ static void XORDrawMoveOrCopyObject(void)
 
 		case LINEPOINT_TYPE:
 		{
-			LineTypePtr		line;
+			LineTypePtr	line;
 			PointTypePtr	point;
 
 			line = (LineTypePtr) Crosshair.AttachedObject.Ptr2;
 			point = (PointTypePtr) Crosshair.AttachedObject.Ptr3;
 			if (point == &line->Point1)
-				XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-					TO_SCREEN_X(point->X +dx), TO_SCREEN_Y(point->Y +dy),
-					TO_SCREEN_X(line->Point2.X), TO_SCREEN_Y(line->Point2.Y));
+				XDrawLine(Dpy, Output.OutputWindow,
+					Crosshair.GC,
+					TO_SCREEN_X(point->X +dx),
+					TO_SCREEN_Y(point->Y +dy),
+					TO_SCREEN_X(line->Point2.X),
+					TO_SCREEN_Y(line->Point2.Y));
 			else
 				XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-					TO_SCREEN_X(point->X +dx), TO_SCREEN_Y(point->Y +dy),
-					TO_SCREEN_X(line->Point1.X), TO_SCREEN_Y(line->Point1.Y));
+					TO_SCREEN_X(point->X +dx),
+					TO_SCREEN_Y(point->Y +dy),
+					TO_SCREEN_X(line->Point1.X),
+					TO_SCREEN_Y(line->Point1.Y));
 			break;
 		}
 
@@ -316,8 +436,8 @@ static void XORDrawMoveOrCopyObject(void)
 		{
 			PolygonTypePtr	polygon;
 			PointTypePtr	point,
-							previous,
-							following;
+					previous,
+					following;
 
 			polygon = (PolygonTypePtr) Crosshair.AttachedObject.Ptr2;
 			point = (PointTypePtr) Crosshair.AttachedObject.Ptr3;
@@ -342,11 +462,15 @@ static void XORDrawMoveOrCopyObject(void)
 
 				/* draw the two segments */
 			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(previous->X), TO_SCREEN_Y(previous->Y),
-				TO_SCREEN_X(point->X +dx), TO_SCREEN_Y(point->Y +dy));
+				TO_SCREEN_X(previous->X),
+				TO_SCREEN_Y(previous->Y),
+				TO_SCREEN_X(point->X +dx),
+				TO_SCREEN_Y(point->Y +dy));
 			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(point->X +dx), TO_SCREEN_Y(point->Y +dy),
-				TO_SCREEN_X(following->X), TO_SCREEN_Y(following->Y));
+				TO_SCREEN_X(point->X +dx),
+				TO_SCREEN_Y(point->Y +dy),
+				TO_SCREEN_X(following->X),
+				TO_SCREEN_Y(following->Y));
 			break;
 		}
 
@@ -356,13 +480,15 @@ static void XORDrawMoveOrCopyObject(void)
 		{
 			TextTypePtr	text = (TextTypePtr) Crosshair.AttachedObject.Ptr2;
 			BoxTypePtr	box = &text->BoundingBox;
-			int			x0, y0;
+			int		x0, y0;
 
 			x0 = box->X1;
 			y0 = Settings.ShowSolderSide ? box->Y2 : box->Y1;
 			XDrawRectangle(Dpy, Output.OutputWindow, Crosshair.GC,
-				TO_SCREEN_X(x0 +dx), TO_SCREEN_Y(y0 +dy),
-				TO_SCREEN(box->X2 -box->X1), TO_SCREEN(box->Y2 -box->Y1));
+				TO_SCREEN_X(x0 +dx),
+				TO_SCREEN_Y(y0 +dy),
+				TO_SCREEN(box->X2 -box->X1),
+				TO_SCREEN(box->Y2 -box->Y1));
 			break;
 		}
 
@@ -381,22 +507,30 @@ static void XORDrawMoveOrCopyObject(void)
 	while (i)
 	{
 		PointTypePtr	point1,
-						point2;;
+				point2;
 
-			/* 'point1' is always the fix-point */
-		if (ptr->MovedPoint == &ptr->Line->Point1)
-		{
-			point1 = &ptr->Line->Point2;
-			point2 = &ptr->Line->Point1;
-		}
-		else
-		{
-			point1 = &ptr->Line->Point1;
-			point2 = &ptr->Line->Point2;
-		}
-		XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(point1->X), TO_SCREEN_Y(point1->Y),
-			TO_SCREEN_X(point2->X +dx), TO_SCREEN_Y(point2->Y +dy));
+		if (ptr->Line->Flags & RUBBERENDFLAG) {
+				/* 'point1' is always the fix-point */
+			if (ptr->MovedPoint == &ptr->Line->Point1) {
+				point1 = &ptr->Line->Point2;
+				point2 = &ptr->Line->Point1;
+			} else {
+				point1 = &ptr->Line->Point1;
+				point2 = &ptr->Line->Point2;
+			}
+			XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+				TO_SCREEN_X(point1->X),
+				TO_SCREEN_Y(point1->Y),
+				TO_SCREEN_X(point2->X +dx),
+				TO_SCREEN_Y(point2->Y +dy));
+                } else
+                        if (ptr->MovedPoint == &ptr->Line->Point1)
+                                XDrawLine(Dpy, Output.OutputWindow, Crosshair.GC,
+                                        TO_SCREEN_X(ptr->Line->Point1.X + dx),
+                                        TO_SCREEN_Y(ptr->Line->Point1.Y + dy),
+                                        TO_SCREEN_X(ptr->Line->Point2.X +dx),
+                                        TO_SCREEN_Y(ptr->Line->Point2.Y +dy));
+ 
 		ptr++;
 		i--;
 	}
@@ -420,7 +554,6 @@ static void DrawAttached(Boolean BlockToo)
 			break;
 			
 			/* the attached line is used by both LINEMODE and POLYGON_MODE */
-		case LINE_MODE:
 		case POLYGON_MODE:
 				/* draw only if starting point is set */
 			if (Crosshair.AttachedLine.State != STATE_FIRST)
@@ -431,8 +564,7 @@ static void DrawAttached(Boolean BlockToo)
 					TO_SCREEN_Y(Crosshair.AttachedLine.Point2.Y));
 
 				/* draw attached polygon only if in POLYGON_MODE */
-			if (Settings.Mode == POLYGON_MODE &&
-				Crosshair.AttachedPolygon.PointN > 1)
+			if (Crosshair.AttachedPolygon.PointN > 1)
 			{
 				CreateTMPPolygon(&Crosshair.AttachedPolygon, 0, 0);
 				XDrawLines(Dpy, Output.OutputWindow, Crosshair.GC,
@@ -441,6 +573,29 @@ static void DrawAttached(Boolean BlockToo)
 			}
 			break;
 
+		case ARC_MODE:
+			if (Crosshair.AttachedBox.State != STATE_FIRST)
+				XORDrawAttachedArc(Settings.LineThickness);
+			break;
+
+		case LINE_MODE:
+				/* draw only if starting point exists */
+			if (Crosshair.AttachedLine.State != STATE_FIRST)
+			{
+				XORDrawAttachedLine(Crosshair.AttachedLine.Point1.X,
+					Crosshair.AttachedLine.Point1.Y,	
+					Crosshair.AttachedLine.Point2.X,	
+					Crosshair.AttachedLine.Point2.Y,	
+					Settings.LineThickness);
+					/* draw two lines ? */
+				if (PCB->Clipping)
+					XORDrawAttachedLine(Crosshair.AttachedLine.Point2.X,
+						Crosshair.AttachedLine.Point2.Y,
+						Crosshair.X, Crosshair.Y,
+						Settings.LineThickness);
+			}
+			break;
+
 		case PASTEBUFFER_MODE:
 			XORDrawBuffer(PASTEBUFFER);
 			break;
@@ -460,9 +615,9 @@ static void DrawAttached(Boolean BlockToo)
 	if (Crosshair.AttachedBox.State == STATE_SECOND ||
 		(BlockToo && Crosshair.AttachedBox.State == STATE_THIRD))
 	{
-		Position    x1, y1,
-					x2, y2,
-					y0;
+		Position	x1, y1,
+				x2, y2,
+				y0;
 
 		x1 = MIN(Crosshair.AttachedBox.Point1.X,Crosshair.AttachedBox.Point2.X);
 		y1 = MIN(Crosshair.AttachedBox.Point1.Y,Crosshair.AttachedBox.Point2.Y);
@@ -470,8 +625,10 @@ static void DrawAttached(Boolean BlockToo)
 		y2 = MAX(Crosshair.AttachedBox.Point1.Y,Crosshair.AttachedBox.Point2.Y);
 		y0 = Settings.ShowSolderSide ? y2 : y1;
 		XDrawRectangle(Dpy, Output.OutputWindow, Crosshair.GC,
-			TO_SCREEN_X(x1), TO_SCREEN_Y(y0),
-			TO_SCREEN(x2-x1), TO_SCREEN(y2-y1));
+			TO_SCREEN_X(x1),
+			TO_SCREEN_Y(y0),
+			TO_SCREEN(x2-x1),
+			TO_SCREEN(y2-y1));
 	}
 }
 
@@ -529,16 +686,22 @@ void RestoreCrosshair(Boolean BlockToo)
  */
 static void FitCrosshairIntoGrid(Position X, Position Y)
 {
-	Position	x1, y1, x2, y2,
-				dx1, dy1, dx2, dy2;
+	Position	x1, y1, x2, y2, height, width,
+			dx1, dy1, dx2, dy2, ox, oy;
 	
+		/* When the widget has just been changed, the Output structure
+		 * can have stale values, so we get the fresh ones
+		 */
+	XtVaGetValues(Output.Output, XtNx, &ox, XtNy, &oy, XtNheight, &height, XtNwidth, &width, NULL);
+	oy = - oy;
+	ox = - ox;
 		/* get PCB coordinates from visible display size.
 		 * If the bottom view mode is active, y2 might be less then y1
 		 */
-	dx1 = TO_PCB_X(Output.OffsetX);
-	dy1 = TO_PCB_Y(Output.OffsetY);
-	dx2 = TO_PCB_X(Output.OffsetX +Output.Width -1);
-	dy2 = TO_PCB_Y(Output.OffsetY +Output.Height -1);
+	dx1 = TO_PCB_X(ox);
+	dy1 = TO_PCB_Y(oy);
+	dx2 = TO_PCB_X(ox +width -1);
+	dy2 = TO_PCB_Y(oy +height -1);
 	x1 = MIN(dx1, dx2);
 	y1 = MIN(dy1, dy2);
 	x2 = MAX(dx1, dx2);
@@ -565,24 +728,24 @@ static void FitCrosshairIntoGrid(Position X, Position Y)
 				 * so we just have to look for it with some integer
 				 * calculations
 				 */
-			Crosshair.X = GRIDFIT_X(x1 +PCB->Grid);
+			Crosshair.X = GRIDFIT_X(x1 +PCB->Grid, PCB->Grid);
 		else
 			Crosshair.X = (x1+x2)/2;
 	}
 	else
 			/* check if the new position matches the grid */
-		Crosshair.X = GRIDFIT_X(Crosshair.X);
+		Crosshair.X = GRIDFIT_X(Crosshair.X, PCB->Grid);
 
-		/* to the same for the second coordinate */
+		/* do the same for the second coordinate */
 	if (Crosshair.Y < y1 || Crosshair.Y > y2)
 	{
 		if (y2 -y1 +1 >= PCB->Grid)
-			Crosshair.Y = GRIDFIT_Y(y1 +PCB->Grid);
+			Crosshair.Y = GRIDFIT_Y(y1 +PCB->Grid, PCB->Grid);
 		else
 			Crosshair.Y = (y1+y2)/2;
 	}
 	else
-		Crosshair.Y = GRIDFIT_Y(Crosshair.Y);
+		Crosshair.Y = GRIDFIT_Y(Crosshair.Y, PCB->Grid);
 }
 
 /* ---------------------------------------------------------------------------
@@ -631,6 +794,7 @@ void InitCrosshair(void)
 		MyFatal("can't create default crosshair GC\n");
 	XSetState(Dpy, Crosshair.GC, Settings.CrosshairColor, Settings.bgColor,
 		GXxor, AllPlanes);
+	XSetLineAttributes(Dpy, Crosshair.GC, 1, LineSolid, CapButt, JoinMiter);
 
 		/* fake an crosshair off entry on stack */
 	CrosshairStackPosition = 0;
@@ -641,6 +805,9 @@ void InitCrosshair(void)
 	Crosshair.MinX = Crosshair.MinY = 0;
 	Crosshair.MaxX = PCB->MaxWidth;
 	Crosshair.MaxY = PCB->MaxHeight;
+	
+		/* clear the mark */
+	Marked.status = False;
 }
 
 /* ---------------------------------------------------------------------------
