@@ -24,7 +24,13 @@
  *
  */
 
-static	char	*rcsid = "$Id: dev_ps.c,v 143.1 1996/09/16 09:08:34 nau Exp $";
+/* Change History:
+ * 10/11/96 11:37 AJF Added support for a Text() driver function.
+ * This was done out of a pressing need to force text to be printed on the
+ * silkscreen layer. Perhaps the design is not the best.
+ */
+
+static	char	*rcsid = "$Id: dev_ps.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 /* PostScript device driver
  * code is shared for EPS and PS output
@@ -52,27 +58,36 @@ static	char	*rcsid = "$Id: dev_ps.c,v 143.1 1996/09/16 09:08:34 nau Exp $";
  * some local prototypes
  */
 static	void	PrintStringArray(char **, int, FILE *);
-static	char	*PS_Init(PrintInitTypePtr, char *);
-static	char	*EPS_Init(PrintInitTypePtr, char *);
-static	char	*PS_EPS_Init(PrintInitTypePtr, char *, Boolean);
+static	char	*PS_Preamble(PrintInitTypePtr, char *);
+static	char	*EPS_Preamble(PrintInitTypePtr, char *);
+static	char	*PS_EPS_Init(PrintInitTypePtr, char *,Boolean);
 static	void	PS_Exit(void);
 static	void	EPS_Exit(void);
+static	void	EPS_Postamble(void);
+static	void	PS_Postamble(void);
 static	void	PS_EPS_Exit(Boolean);
-static	void	PS_SetColor(XColor);
-static	void	PS_PrintLayer(LayerTypePtr, int);
-static	void	PS_PrintLine(LineTypePtr);
-static	void	PS_PrintPolygon(PolygonTypePtr);
-static	void	PS_PrintText(TextTypePtr);
-static	void	PS_PrintElementPackage(ElementTypePtr);
-static	void	PS_PrintPad(PadTypePtr);
-static	void	PS_PrintPinOrVia(PinTypePtr);
-static	void	PS_PrintClearPinOrViaOnGroundplane(PinTypePtr);
-static	void	PS_PrintPadMask(PadTypePtr);
-static	void	PS_PrintPinOrViaMask(PinTypePtr);
-static	void	PS_FilledRectangle(Position, Position, Position, Position);
-static	void	PS_Outline(Position, Position, Position, Position);
-static	void	PS_Alignment(Position, Position, Position, Position);
-static	void	PS_DrillHelper(PinTypePtr);
+static	void	PS_Init(PrintInitTypePtr);
+static	void	EPS_Init(PrintInitTypePtr);
+static	void	PS_SetColor(XColor, int);
+static	void	PS_PrintLayer(LayerTypePtr, int, Boolean, int);
+static	void	PS_PrintLine(LineTypePtr, int);
+static	void	PS_PrintArc(ArcTypePtr, int);
+static	void	PS_PrintPolygon(PolygonTypePtr, int);
+static	void	PS_PrintText(TextTypePtr, int);
+static	void	PS_PrintTextLowLevel(TextTypePtr, int);
+static	void	PS_PrintElementPackage(ElementTypePtr, int);
+static	void	PS_PrintPad(PadTypePtr, int);
+static	void	PS_PrintPinOrVia(PinTypePtr, int);
+static	void	PS_PrintClearPinOrViaOnGroundplane(PinTypePtr, int);
+static	void	PS_PrintPadMask(PadTypePtr, int);
+static	void	PS_PrintPinOrViaMask(PinTypePtr, int);
+static	void	PS_FilledRectangle(Position, Position,
+			Position, Position, int);
+static	void	PS_Outline(Position, Position,
+			Position, Position, int);
+static	void	PS_Alignment(Position, Position,
+			Position, Position, int);
+static	void	PS_DrillHelper(PinTypePtr, int);
 
 /* ----------------------------------------------------------------------
  * some local identifiers
@@ -82,9 +97,11 @@ static	void	PS_DrillHelper(PinTypePtr);
  *   A:         draws an element arc
  *   Alignment: draws the boards outline
  *   B:         draws a filled box
+ *   CLRB       clears a box
  *   CLRPA:     clears a pad
- *   CLRPV:     clears a circle
+ *   CLRPV:     clears an octagon 
  *   CLRPVSQ:   clears a square
+ *   CLRPVR:    clears a circle
  *   DH:        drill helper, a small circle
  *   FILL:      draws a filled rectangle for the ground plane
  *   L:         draws a line
@@ -92,7 +109,8 @@ static	void	PS_DrillHelper(PinTypePtr);
  *   P:         draws a filled pin-polygon 
  *   PA:        draws a pad
  *   PO:        draws a filled polygon 
- *   PV:        draws a pin or via
+ *   PV:        draws an octagonal pin or via
+ *   PVR:       draws a round pin or via
  *   PVSQ:      draws a square pin or via
  * additional information is available from the PS comments.
  *
@@ -100,54 +118,68 @@ static	void	PS_DrillHelper(PinTypePtr);
  * the author
  */
 static	PrintDeviceType	PS_QueryConstants = {
-			"PostScript",
-			"ps",
-			PS_Init,
-			PS_Exit,
-			PS_SetColor,
-			NULL,				/* no drill information */
-			PS_PrintLayer,
-			PS_PrintElementPackage,
-			PS_PrintPad,
-			PS_PrintPinOrVia,
-			PS_PrintPadMask,
-			PS_PrintPinOrViaMask,
-			PS_PrintClearPinOrViaOnGroundplane,
-			PS_FilledRectangle,
-			PS_Outline,
-			PS_Alignment,
-			PS_DrillHelper,
-			True,				/* driver can handle color */
-			True,				/* and inverted output */
-			True,				/* handles ground planes */
-			False,				/* handles no drill information */
-			True,				/* handles mask layers */
-			True },				/* handles different media */
+	"PostScript",
+	"ps",
+	PS_Init,
+	PS_Exit,
+	PS_Preamble,
+	PS_Postamble,
+	PS_SetColor,
+	NULL,				/* no drill information */
+	PS_PrintLayer,
+	PS_PrintElementPackage,
+	PS_PrintText,
+	PS_PrintPad,
+	PS_PrintPinOrVia,
+	PS_PrintPadMask,
+	PS_PrintPinOrViaMask,
+	PS_PrintClearPinOrViaOnGroundplane,
+	PS_FilledRectangle,
+	PS_Outline,
+	PS_Alignment,
+	PS_DrillHelper,
+	True,				/* driver can handle color */
+	True,				/* and inverted output */
+	True,				/* handles ground planes */
+	False,				/* handles no drill information */
+	True,				/* handles mask layers */
+	True,				/* handles different media */
+	True,				/* allows mirroring */
+	True,				/* allows rotate */
+	False,				/* needs preprint calls */
+	False },			/* needs an AppMap */
 
-						EPS_QueryConstants = {
-			"encapsulated PostScript",
-			"eps",
-			EPS_Init,
-			EPS_Exit,
-			PS_SetColor,
-			NULL,				/* no drill information */
-			PS_PrintLayer,
-			PS_PrintElementPackage,
-			PS_PrintPad,
-			PS_PrintPinOrVia,
-			PS_PrintPadMask,
-			PS_PrintPinOrViaMask,
-			PS_PrintClearPinOrViaOnGroundplane,
-			PS_FilledRectangle,
-			PS_Outline,
-			PS_Alignment,
-			PS_DrillHelper,
-			True,
-			True,
-			True,
-			False,
-			True,
-			False};
+			EPS_QueryConstants = {
+	"encapsulated PostScript",
+	"eps",
+	EPS_Init,
+	EPS_Exit,
+	EPS_Preamble,
+	EPS_Postamble,
+	PS_SetColor,
+	NULL,				/* no drill information */
+	PS_PrintLayer,
+	PS_PrintElementPackage,
+	PS_PrintText,
+	PS_PrintPad,
+	PS_PrintPinOrVia,
+	PS_PrintPadMask,
+	PS_PrintPinOrViaMask,
+	PS_PrintClearPinOrViaOnGroundplane,
+	PS_FilledRectangle,
+	PS_Outline,
+	PS_Alignment,
+	PS_DrillHelper,
+	True,
+	True,
+	True,
+	False,
+	True,
+	False,		/* encapsulated doesn't allow media changes */
+	True,
+	True,				/* allows rotate */
+	False,		/* needs preprint calls */
+	False};
 
 static	PrintInitType	PS_Flags;
 
@@ -199,6 +231,17 @@ static	char	*PS_Functions[] = {
 	"   grestore",
 	"} def",
 	"",
+	"/PVR {",
+	"% pin or via, x, y and thickness are on the stack",
+	"	/drillinghole exch def /thickness exch def /y exch def /x exch def",
+	"	gsave 0 setlinewidth",
+	"	newpath x y thickness 2 div 0 360 arc closepath fill stroke",
+	"% draw drilling whole",
+	"	White",
+	"	newpath x y drillinghole 2 div 0 360 arc closepath fill stroke",
+	"	grestore",
+	"} def",
+	"",
 	"/PVSQ {",
 	"% square pin or via, x, y and thickness are on the stack",
 	"	/drillinghole exch def /thickness exch def /y exch def /x exch def",
@@ -269,10 +312,29 @@ static	char	*PS_Functions[] = {
 	"	grestore",
 	"} def",
 	"",
+	"/CLRPVR {",
+	"% clears a round pin/via for groundplane; x,y and thickness are on the stack",
+	"	/thickness exch def /y exch def /x exch def",
+	"	gsave White 0 setlinewidth",
+	"	newpath x y thickness 2 div 0 360 arc closepath fill stroke",
+	"	grestore",
+	"} def",
+	"",
 	"/CLRPA {",
-	"% clears a pad, x1, y1, x2, y2 abd thickness are on stack",
+	"% clear line, get x1, y1, x2, y2 and thickness from stack",
 	"	/thick exch def /y2 exch def /x2 exch def /y1 exch def /x1 exch def",
-	"	gsave White x1 y1 x2 y2 thick PA grestore",
+	"	gsave White thick setlinewidth",
+	"	x1 y1 moveto x2 y2 lineto stroke",
+	"	grestore",
+	"} def",
+	"",
+	"/CLRB {",
+	"% cleared box, get x1, y1, x2 and y2 from stack",
+	"	/y2 exch def /x2 exch def /y1 exch def /x1 exch def",
+	"	gsave White newpath",
+	"	x1 y1 moveto x2 y1 lineto x2 y2 lineto x1 y2 lineto",
+	"	closepath fill stroke",
+	"	grestore",
 	"} def",
 	"",
 	"/FILL {",
@@ -354,18 +416,24 @@ PrintDeviceTypePtr EPS_Query(void)
 /* ----------------------------------------------------------------------
  * call init routine for PostScript output
  */
-static char *PS_Init(PrintInitTypePtr Flags, char *Description)
+static char *PS_Preamble(PrintInitTypePtr Flags, char *Description)
 {
 	return(PS_EPS_Init(Flags, Description, False));
 }
+static void PS_Init(PrintInitTypePtr wasted)
+{
+}
 
 /* ----------------------------------------------------------------------
  * call init routine for encapsulated PostScript output
  */
-static char *EPS_Init(PrintInitTypePtr Flags, char *Description)
+static char *EPS_Preamble(PrintInitTypePtr Flags, char *Description)
 {
 	return(PS_EPS_Init(Flags, Description, True));
 }
+static void EPS_Init(PrintInitTypePtr wasted)
+{
+}
 
 /* ----------------------------------------------------------------------
  * prints PostScript or enc. PostScript header with function definition
@@ -374,10 +442,10 @@ static char *EPS_Init(PrintInitTypePtr Flags, char *Description)
 static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 	Boolean CreateEPS)
 {
-			BoxType		box;
-			time_t		currenttime;
-			float		dx, dy;
-	struct	passwd		*pwentry;
+	BoxType		box;
+	time_t		currenttime;
+	float		dx, dy;
+	struct	passwd	*pwentry;
 
 		/* save passed-in data */
 	PS_Flags = *Flags;
@@ -395,14 +463,20 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 	dx = dy = PS_Flags.Scale;
 	dx *= (float) (PS_Flags.BoundingBox.X2 -PS_Flags.BoundingBox.X1);
 	dy *= (float) (PS_Flags.BoundingBox.Y2 -PS_Flags.BoundingBox.Y1);
-	PS_Flags.OffsetY = PS_Flags.SelectedMedia->Height -PS_Flags.OffsetY;
-	PS_Flags.OffsetY -= (Dimension) (PS_Flags.RotateFlag ? dx : dy);
 
 		/* create standard PS header */
 	if (CreateEPS)
+	{
+		PS_Flags.OffsetX = 0;
+		PS_Flags.OffsetY = 0;
 		fputs("%!PS-Adobe-3.0 EPSF-3.0\n", PS_Flags.FP);
+	}
 	else
+	{
+		PS_Flags.OffsetY = PS_Flags.SelectedMedia->Height -PS_Flags.OffsetY;
+		PS_Flags.OffsetY -= (Dimension) (PS_Flags.RotateFlag ? dx : dy);
 		fputs("%!PS-Adobe-3.0\n", PS_Flags.FP);
+	}
 	fprintf(PS_Flags.FP, "%%%%Title: %s, %s\n", UNKNOWN(PCB->Name),
 		UNKNOWN(Description));
 	fprintf(PS_Flags.FP, "%%%%Creator: %s "RELEASE"\n", Progname);
@@ -432,7 +506,9 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 		box.Y2= (Position)((dx +PS_Flags.OffsetY) *PS_UNIT) +1;
 	}
 
-		/* print it if encapsulated PostScript has been requested */
+		/* print it if encapsulated PostScript has been requested
+		 * and add the appropriate structured comments
+		 */
 	if (CreateEPS)
 	{
 		fprintf(PS_Flags.FP, "%%%%BoundingBox: %i %i %i %i\n",
@@ -443,11 +519,11 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 	{
 		fputs("%%Pages: 1\n", PS_Flags.FP);
 		fputs("%%PageOrder: Ascend\n", PS_Flags.FP);
+		fprintf(PS_Flags.FP, "%%%%IncludeFeature: *PageSize %s\n",
+			PS_Flags.SelectedMedia->Name);
 	}
 
 		/* OK, continue with structured comments */
-	fprintf(PS_Flags.FP, "%%%%IncludeFeature: *PageSize %s\n",
-		PS_Flags.SelectedMedia->Name);
 	fputs("%%EndComments\n", PS_Flags.FP);
 	fputs("%%BeginProlog\n", PS_Flags.FP);
 	PrintStringArray(PS_Functions, ENTRIES(PS_Functions), PS_Flags.FP);
@@ -485,10 +561,14 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 
 		/* add information about layout size, offset ... */
 	fprintf(PS_Flags.FP, "%% PCBMIN(%d,%d), PCBMAX(%d,%d)\n",
-		(int) PS_Flags.BoundingBox.X1, (int) PS_Flags.BoundingBox.Y1,
-		(int) PS_Flags.BoundingBox.X2, (int) PS_Flags.BoundingBox.Y2);
+		(int) PS_Flags.BoundingBox.X1,
+		(int) PS_Flags.BoundingBox.Y1,
+		(int) PS_Flags.BoundingBox.X2,
+		(int) PS_Flags.BoundingBox.Y2);
 	fprintf(PS_Flags.FP, "%% PCBOFFSET(%d,%d), PCBSCALE(%.5f)\n",
-		PS_Flags.OffsetX, PS_Flags.OffsetY, PS_Flags.Scale);
+		PS_Flags.OffsetX,
+		PS_Flags.OffsetY,
+		PS_Flags.Scale);
 	fputs("% PCBSTARTDATA --- do not remove ---\n", PS_Flags.FP);
 	fputs("gsave\n", PS_Flags.FP);
 
@@ -504,9 +584,11 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 	fprintf(PS_Flags.FP, "%.5f %.5f scale\n",
 		PS_UNIT, PS_UNIT);
 	fprintf(PS_Flags.FP, "%i %i translate\n",
-		PS_Flags.OffsetX, PS_Flags.OffsetY);
+		PS_Flags.OffsetX,
+		PS_Flags.OffsetY);
 	fprintf(PS_Flags.FP, "%.3f %.3f scale\n",
-		PS_Flags.Scale, PS_Flags.Scale);
+		PS_Flags.Scale,
+		PS_Flags.Scale);
 	if (PS_Flags.RotateFlag)
 	{
 		fprintf(PS_Flags.FP, "%i 0 translate\n",
@@ -519,7 +601,8 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 			(int) (PS_Flags.BoundingBox.Y2 -PS_Flags.BoundingBox.Y1));
 		fputs("1 -1 scale\n", PS_Flags.FP);
 	}
-	fprintf(PS_Flags.FP, "%i %i translate\n", -PS_Flags.BoundingBox.X1,
+	fprintf(PS_Flags.FP, "%i %i translate\n",
+		-PS_Flags.BoundingBox.X1,
 		-PS_Flags.BoundingBox.Y1);
 	return(NULL);
 }
@@ -527,18 +610,24 @@ static char *PS_EPS_Init(PrintInitTypePtr Flags, char *Description,
 /* ----------------------------------------------------------------------
  * call exit routine for PostScript output
  */
-static void PS_Exit(void)
+static void PS_Postamble(void)
 {
 	PS_EPS_Exit(False);
 }
+static void PS_Exit(void)
+{
+}
 
 /* ----------------------------------------------------------------------
  * call exit routine for encapsulated PostScript output
  */
-static void EPS_Exit(void)
+static void EPS_Postamble(void)
 {
 	PS_EPS_Exit(True);
 }
+static void EPS_Exit(void)
+{
+}
 
 /* ---------------------------------------------------------------------------
  * exit code for this driver is empty
@@ -556,30 +645,75 @@ static void PS_EPS_Exit(Boolean CreateEPS)
 /* ----------------------------------------------------------------------
  * prints layer data
  */
-static void PS_PrintLayer(LayerTypePtr Layer, int GroupNumber)
+static void PS_PrintLayer(LayerTypePtr Layer, int GroupNumber,
+	Boolean SilkscreenTextFlag, int unused)
 {
+        int ThermLayerFlag, LayerPIPFlag;
+
 	fprintf(PS_Flags.FP, "%% LayerGroup: %i, Layer: \"%s\" (#%i)\n",
-		GroupNumber, UNKNOWN(Layer->Name), GetLayerNumber(PCB->Data, Layer));
-	LINE_LOOP(Layer, PS_PrintLine(line));
-	TEXT_LOOP(Layer, PS_PrintText(text));
-	POLYGON_LOOP(Layer, PS_PrintPolygon(polygon));
+		GroupNumber,
+		UNKNOWN(Layer->Name),
+		GetLayerNumber(PCB->Data, Layer));
+	LINE_LOOP(Layer, PS_PrintLine(line, unused));
+	ARC_LOOP(Layer, PS_PrintArc(arc, unused));
+        TEXT_LOOP(Layer, {
+                if (!TEST_FLAG(ONSILKFLAG, text))
+                        PS_PrintTextLowLevel(text, 0);
+        });
+        POLYGON_LOOP(Layer, PS_PrintPolygon(polygon, 0));
+          ThermLayerFlag = L0THERMFLAG << GetLayerNumber(PCB->Data, Layer);
+          LayerPIPFlag = L0PIPFLAG << GetLayerNumber(PCB->Data, Layer);
+          ALLPIN_LOOP(PCB->Data,
+                  if (TEST_FLAG(LayerPIPFlag, pin) && !TEST_FLAG(HOLEFLAG, pin))
+                {
+                        if (TEST_FLAG(ThermLayerFlag, pin))
+                                SET_FLAG(USETHERMALFLAG, pin);
+                        PS_PrintClearPinOrViaOnGroundplane(pin, 0);
+                }
+          );
+          VIA_LOOP(PCB->Data,
+                  if (TEST_FLAG(LayerPIPFlag, via) && !TEST_FLAG(HOLEFLAG, via))
+                {
+                        if (TEST_FLAG(ThermLayerFlag, via))
+                                SET_FLAG(USETHERMALFLAG, via);
+                        PS_PrintClearPinOrViaOnGroundplane(via, 0);
+                }
+          );
+ 
 }
 
 /* ----------------------------------------------------------------------
  * prints a line
  */
-static void PS_PrintLine(LineTypePtr Line)
+static void PS_PrintLine(LineTypePtr Line, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
-		(int) Line->Point1.X, (int) Line->Point1.Y,
-		(int) Line->Point2.X, (int) Line->Point2.Y,
+		(int) Line->Point1.X,
+		(int) Line->Point1.Y,
+		(int) Line->Point2.X,
+		(int) Line->Point2.Y,
 		(int) Line->Thickness);
 }
 
+/* ---------------------------------------------------------------------------
+ * prints an arc 
+ */
+static void PS_PrintArc(ArcTypePtr arc, int unused)
+{
+	fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d A\n",
+		(int) arc->X,
+		(int) arc->Y,
+		(int) arc->Width,
+		(int) arc->Height,
+		(int) arc->Thickness,
+		arc->StartAngle,
+		arc->Delta);
+}
+
 /* ---------------------------------------------------------------------------
  * prints a filled polygon
  */
-static void PS_PrintPolygon(PolygonTypePtr Ptr)
+static void PS_PrintPolygon(PolygonTypePtr Ptr, int unused)
 {
 	int i = 0;
 
@@ -588,30 +722,31 @@ static void PS_PrintPolygon(PolygonTypePtr Ptr)
 			if (i++ % 9 == 8)
 				fputc('\n', PS_Flags.FP);
 			fprintf(PS_Flags.FP, "%i %i ",
-				(int) point->X, (int) point->Y);
+				(int) point->X,
+				(int) point->Y);
 		}
 	);
 	fprintf(PS_Flags.FP, "%d PO\n", Ptr->PointN);
 }
 
 /* ----------------------------------------------------------------------
- * prints a text
+ * lowlevel routine to print a text
  * the routine is identical to DrawText() in module draw.c except
  * that DrawLine() and DrawRectangle() are replaced by their corresponding
  * printing routines
  */
-static void PS_PrintText(TextTypePtr Text)
+static void PS_PrintTextLowLevel(TextTypePtr Text, int unused)
 {
 	Position		x = 0,
-					width;
+				width;
 	unsigned char	*string = (unsigned char *) Text->TextString;
 	Cardinal		n;
 	FontTypePtr		font = &PCB->Font;
 
 		/* get the center of the text for mirroring */
 	width = Text->Direction & 0x01 ? 
-				Text->BoundingBox.Y2 -Text->BoundingBox.Y1 :
-				Text->BoundingBox.X2 -Text->BoundingBox.X1;
+		Text->BoundingBox.Y2 -Text->BoundingBox.Y1 :
+		Text->BoundingBox.X2 -Text->BoundingBox.X1;
 	while (string && *string)
 	{
 			/* draw lines if symbol is valid and data is present */
@@ -630,12 +765,6 @@ static void PS_PrintText(TextTypePtr Text)
 				newline.Point2.Y = newline.Point2.Y      *Text->Scale /100;
 				newline.Thickness = newline.Thickness *Text->Scale /100;
 
-					/* do some mirroring, swaping and rotating */
-				if (TEST_FLAG(MIRRORFLAG, Text))
-				{
-					newline.Point1.X = width -newline.Point1.X;
-					newline.Point2.X = width -newline.Point2.X;
-				}
 				RotateLineLowLevel(&newline, 0, 0, Text->Direction);
 
 					/* the labels of SMD objects on the bottom
@@ -653,7 +782,7 @@ static void PS_PrintText(TextTypePtr Text)
 				newline.Point1.Y += Text->Y;
 				newline.Point2.X += Text->X;
 				newline.Point2.Y += Text->Y;
-				PS_PrintLine(&newline);
+				PS_PrintLine(&newline, unused);
 			}
 
 				/* move on to next cursor position */
@@ -670,12 +799,6 @@ static void PS_PrintText(TextTypePtr Text)
 			defaultsymbol.X2 = (defaultsymbol.X2 +x) *Text->Scale /100;
 			defaultsymbol.Y2 = defaultsymbol.Y2      *Text->Scale /100;
 
-				/* do some mirroring ... */
-			if (TEST_FLAG(MIRRORFLAG, Text))
-			{
-				defaultsymbol.X1 = width -defaultsymbol.X1;
-				defaultsymbol.X2 = width -defaultsymbol.X2;
-			}
 			RotateBoxLowLevel(&defaultsymbol, 0, 0, Text->Direction);
 
 				/* add offset and draw box */
@@ -684,8 +807,10 @@ static void PS_PrintText(TextTypePtr Text)
 			defaultsymbol.X2 += Text->X;
 			defaultsymbol.Y2 += Text->Y;
 			fprintf(PS_Flags.FP, "%d %d %d %d B\n",
-				(int) defaultsymbol.X1, (int) defaultsymbol.Y1,
-				(int) defaultsymbol.X2, (int) defaultsymbol.Y2);
+				(int) defaultsymbol.X1,
+				(int) defaultsymbol.Y1,
+				(int) defaultsymbol.X2,
+				(int) defaultsymbol.Y2);
 
 				/* move on to next cursor position */
 			x += size;
@@ -694,120 +819,188 @@ static void PS_PrintText(TextTypePtr Text)
 	}
 }
 
+/* ----------------------------------------------------------------------
+ * print text; the code has been added by
+ * Albert John FitzPatrick III <ajf_nylorac@acm.org>
+ * see ../CHANGES for details
+ */
+static void PS_PrintText(TextTypePtr Text, int unused)
+{
+		/* FIXME: This code has not been tested. */
+	PS_PrintTextLowLevel(Text, unused);
+}
+
 /* ----------------------------------------------------------------------
  * prints package outline
  */
-static void PS_PrintElementPackage(ElementTypePtr Element)
+static void PS_PrintElementPackage(ElementTypePtr Element, int unused)
 {
-	ELEMENTLINE_LOOP(Element, PS_PrintLine(line););
+	ELEMENTLINE_LOOP(Element, PS_PrintLine(line, unused););
 	ARC_LOOP(Element, 
 		fprintf(PS_Flags.FP, "%d %d %d %d %d %d %d A\n",
-			(int) arc->X, (int) arc->Y,
-			(int) arc->Width, (int) arc->Height,
+			(int) arc->X,
+			(int) arc->Y,
+			(int) arc->Width,
+			(int) arc->Height,
 			(int) arc->Thickness,
 			arc->StartAngle,
 			arc->Delta);
 	);
-	PS_PrintText(&ELEMENT_TEXT(PCB, Element));
+	if (!TEST_FLAG(HIDENAMEFLAG, Element))
+		PS_PrintTextLowLevel(&ELEMENT_TEXT(PCB, Element), unused);
 }
 
 /* ----------------------------------------------------------------------
  * prints a pad
  */
-static void PS_PrintPad(PadTypePtr Pad)
+static void PS_PrintPad(PadTypePtr Pad, int unused)
 {
+	if (TEST_FLAG(SQUAREFLAG, Pad))
+	{
+		Dimension scrunch = Pad->Thickness/2;
+	fprintf(PS_Flags.FP, "%d %d %d %d B\n",
+		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point2.X  : Pad->Point1.X) - scrunch,
+		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point2.Y  : Pad->Point1.Y) - scrunch,
+		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point1.X  : Pad->Point2.X) + scrunch,
+		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point1.Y  : Pad->Point2.Y) + scrunch);
+	}
+	else
 	fprintf(PS_Flags.FP, "%d %d %d %d %d PA\n",
-		(int) Pad->Point1.X, (int) Pad->Point1.Y,
-		(int) Pad->Point2.X, (int) Pad->Point2.Y,
+		(int) Pad->Point1.X,
+		(int) Pad->Point1.Y,
+		(int) Pad->Point2.X,
+		(int) Pad->Point2.Y,
 		(int) Pad->Thickness);
 }
 
 /* ----------------------------------------------------------------------
  * prints a via or pin
  */
-static void PS_PrintPinOrVia(PinTypePtr Ptr)
+static void PS_PrintPinOrVia(PinTypePtr Ptr, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d %s\n",
-		(int) Ptr->X, (int) Ptr->Y,
-		(int) Ptr->Thickness, (int) Ptr->DrillingHole,
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "PVSQ" : "PV");
+		(int) Ptr->X,
+		(int) Ptr->Y,
+		(int) Ptr->Thickness,
+		(int) Ptr->DrillingHole,
+		TEST_FLAG(SQUAREFLAG, Ptr) ? "PVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "PV" : "PVR");
 }
 
 /* ----------------------------------------------------------------------
  * clears the area around a via or pin on the groundplane
  */
-static void PS_PrintClearPinOrViaOnGroundplane(PinTypePtr Ptr)
+static void PS_PrintClearPinOrViaOnGroundplane(PinTypePtr Ptr, int unused)
 {
+        Dimension       half = Ptr->Thickness/2 + GROUNDPLANEFRAME;
+
 	fprintf(PS_Flags.FP, "%d %d %d %s\n",
-		(int) Ptr->X, (int) Ptr->Y,
-		TEST_FLAG(ONGROUNDPLANEFLAG, Ptr) ?
-			(int) Ptr->DrillingHole :
-			(int) Ptr->Thickness +2 *GROUNDPLANEFRAME,
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : "CLRPV");
+		(int) Ptr->X,
+		(int) Ptr->Y,
+		(int) (Ptr->Thickness +2 *GROUNDPLANEFRAME),
+		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "CLRPV" : "CLRPVR");
+        if (TEST_FLAG(USETHERMALFLAG, Ptr)) {
+                if (!TEST_FLAG(SQUAREFLAG, Ptr))
+                        half = (7 * half) / 10;
+                fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
+                        Ptr->X - half, Ptr->Y - half,
+                        Ptr->X + half, Ptr->Y + half,
+                        GROUNDPLANEFRAME);
+                fprintf(PS_Flags.FP, "%d %d %d %d %d L\n",
+                        Ptr->X - half, Ptr->Y + half,
+                        Ptr->X + half, Ptr->Y - half,
+                        GROUNDPLANEFRAME);
+                CLEAR_FLAG(USETHERMALFLAG, Ptr);
+        }
 }
 
 /* ----------------------------------------------------------------------
  * prints a pad mask
  */
-static void PS_PrintPadMask(PadTypePtr Pad)
+static void PS_PrintPadMask(PadTypePtr Pad, int unused)
 {
+	if (TEST_FLAG(SQUAREFLAG, Pad))
+	{
+		Dimension scrunch = Pad->Thickness/2 + GROUNDPLANEFRAME;
+	fprintf(PS_Flags.FP, "%d %d %d %d CLRB\n",
+		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point2.X  : Pad->Point1.X) - scrunch,
+		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point2.Y  : Pad->Point1.Y) - scrunch,
+		((Pad->Point1.X > Pad->Point2.X) ? Pad->Point1.X  : Pad->Point2.X) + scrunch,
+		((Pad->Point1.Y > Pad->Point2.Y) ? Pad->Point1.Y  : Pad->Point2.Y) + scrunch);
+	}
+	else
 	fprintf(PS_Flags.FP, "%d %d %d %d %d CLRPA\n",
-		(int) Pad->Point1.X, (int) Pad->Point1.Y,
-		(int) Pad->Point2.X, (int) Pad->Point2.Y,
-		(int) Pad->Thickness);
+		(int) Pad->Point1.X,
+		(int) Pad->Point1.Y,
+		(int) Pad->Point2.X,
+		(int) Pad->Point2.Y,
+		(int) Pad->Thickness + 2*GROUNDPLANEFRAME);
 }
 
 /* ----------------------------------------------------------------------
  * prints a via or pin mask
  */
-static void PS_PrintPinOrViaMask(PinTypePtr Ptr)
+static void PS_PrintPinOrViaMask(PinTypePtr Ptr, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %s\n",
-		(int) Ptr->X, (int) Ptr->Y,
-		(int) Ptr->Thickness,
-		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : "CLRPV");
+		(int) Ptr->X,
+		(int) Ptr->Y,
+		(int) Ptr->Thickness + 2*GROUNDPLANEFRAME,
+		TEST_FLAG(SQUAREFLAG, Ptr) ? "CLRPVSQ" : TEST_FLAG(OCTAGONFLAG, Ptr) ? "CLRPV" : "CLRPVR");
 }
 
 /* ---------------------------------------------------------------------------
  * draws a filled rectangle for the ground plane
  */
 static void PS_FilledRectangle(Position X1, Position Y1,
-	Position X2, Position Y2)
+	Position X2, Position Y2, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d FILL\n",
-		(int) X1, (int) Y1, (int) X2, (int) Y2);
+		(int) X1,
+		(int) Y1,
+		(int) X2,
+		(int) Y2);
 }
 
 /* ---------------------------------------------------------------------------
  * draw the outlines of a layout;
  * the upper/left and lower/right corner are passed
  */
-static void PS_Outline(Position X1, Position Y1, Position X2, Position Y2)
+static void PS_Outline(Position X1, Position Y1,
+	Position X2, Position Y2, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d Outline\n",
-		(int) X1, (int) Y1, (int) X2, (int) Y2);
+		(int) X1,
+		(int) Y1,
+		(int) X2,
+		(int) Y2);
 }
 
 /* ---------------------------------------------------------------------------
  * draw the alignment targets;
  * the upper/left and lower/right corner are passed
  */
-static void PS_Alignment(Position X1, Position Y1, Position X2, Position Y2)
+static void PS_Alignment(Position X1, Position Y1,
+	Position X2, Position Y2, int unused)
 {
 	fprintf(PS_Flags.FP, "%d %d %d %d %d Alignment\n",
-		(int) X1, (int) Y1, (int) X2, (int) Y2,
+		(int) X1,
+		(int) Y1,
+		(int) X2,
+		(int) Y2,
 		(int) Settings.AlignmentDistance);
 }
 
 /* ----------------------------------------------------------------------
  * prints a via or pin
  */
-static void PS_DrillHelper(PinTypePtr Ptr)
+static void PS_DrillHelper(PinTypePtr Ptr, int unused)
 {
 	if (Ptr->DrillingHole >= 4*MIN_PINORVIAHOLE)
 		fprintf(PS_Flags.FP, "%d %d %d %d DH\n",
-			(int) Ptr->X, (int) Ptr->Y,
-			(int) 2*MIN_PINORVIAHOLE, (int) MIN_PINORVIAHOLE);
+			(int) Ptr->X,
+			(int) Ptr->Y,
+			(int) 2*MIN_PINORVIAHOLE,
+			(int) MIN_PINORVIAHOLE);
 }
 
 /* ----------------------------------------------------------------------
@@ -815,7 +1008,7 @@ static void PS_DrillHelper(PinTypePtr Ptr)
  * set PostScript identifier to the calculated value or to black by 
  * default
  */
-static void PS_SetColor(XColor RGB)
+static void PS_SetColor(XColor RGB, int unused)
 {
 	fprintf(PS_Flags.FP,
 		"/Color {%.3f %.3f %.3f mysetrgbcolor} def Color\n",
