@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: file.c,v 143.1 1996/09/16 09:08:36 nau Exp $";
+static	char	*rcsid = "$Id: file.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 /* file save, load, merge ... routines
  * getpid() needs a cast to (int) to get rid of compiler warnings
@@ -74,9 +74,9 @@ static	void	WritePCBFontData(FILE *);
 static	void	WritePCBViaData(FILE *);
 static	void	WritePCBElementData(FILE *);
 static	void	WritePCBLayerData(FILE *, Cardinal);
-static	int		WritePCB(FILE *);
-static	int		WritePCBFile(char *);
-static	int		WritePCBPipe(char *);
+static	int	WritePCB(FILE *);
+static	int	WritePCBFile(char *);
+static	int	WritePCBPipe(char *);
 
 /* ---------------------------------------------------------------------------
  * opens a file and check if it exists
@@ -123,7 +123,7 @@ FILE *OpenConnectionDataFile(void)
  */
 int SavePCB(char *Filename)
 {
-	int		retcode;
+	int	retcode;
 	char	*copy;
 
 	if (!(retcode = WritePCBPipe(Filename)))
@@ -148,7 +148,7 @@ int SavePCB(char *Filename)
 int LoadPCB(char *Filename)
 {
 	PCBTypePtr	newPCB = CreateNewPCB(False);
-	int			save;
+	int		save;
 
 		/* new data isn't added to the undo list */
 	if (!ParsePCB(newPCB, Filename))
@@ -212,8 +212,8 @@ static void WritePCBInfoHeader(FILE *FP)
 {
 	struct passwd	*pwentry;
 	struct hostent	*hostentry;
-	time_t			currenttime;
-	char			hostname[256];
+	time_t		currenttime;
+	char		hostname[256];
 
 		/* write some useful comments */
 	currenttime = time(NULL);
@@ -236,7 +236,7 @@ static void WritePCBInfoHeader(FILE *FP)
 static void WritePCBDataHeader(FILE *FP)
 {
 	Cardinal	group,
-				entry;
+			entry;
 
 	fputs("\nPCB(", FP);
 	PrintQuotedString(FP, EMPTY(PCB->Name));
@@ -246,7 +246,7 @@ static void WritePCBDataHeader(FILE *FP)
 		(int) PCB->Grid, (int) PCB->GridOffsetX, (int) PCB->GridOffsetY);
 	fprintf(FP, "Cursor(%i %i %i)\n",
 		(int) Crosshair.X, (int) Crosshair.Y, PCB->Zoom);
-	fprintf(FP, "Flags(0x%04x)\n", (int) PCB->Flags);
+	fprintf(FP, "Flags(0x%08x)\n", (int) PCB->Flags);
 	fputs("Groups(\"", FP);
 	for (group = 0; group < MAX_LAYER; group++)
 		if (PCB->LayerGroups.Number[group])
@@ -318,7 +318,7 @@ static void WritePCBViaData(FILE *FP)
 			(int) via->X, (int) via->Y,
 			(int) via->Thickness, (int) via->DrillingHole);
 		PrintQuotedString(FP, EMPTY(via->Name));
-		fprintf(FP, " 0x%04x)\n", (int) via->Flags);
+		fprintf(FP, " 0x%08x)\n", (int) via->Flags);
 
 	);
 
@@ -338,25 +338,25 @@ static void WritePCBElementData(FILE *FP)
 				/* the coordinates and text-flags are the same for
 				 * both names of an element
 				 */
-		fprintf(FP, "\nElement(0x%04x ", (int) element->Flags);
+		fprintf(FP, "\nElement(0x%08x ", (int) element->Flags);
 		PrintQuotedString(FP, EMPTY(DESCRIPTION_NAME(element)));
 		fputc(' ', FP);
 		PrintQuotedString(FP, EMPTY(NAMEONPCB_NAME(element)));
 		fputc(' ', FP);
 		PrintQuotedString(FP, EMPTY(VALUE_NAME(element)));
-		fprintf(FP, " %i %i %i %i 0x%04x)\n(\n",
+		fprintf(FP, " %i %i %i %i 0x%08x)\n(\n",
 				(int) DESCRIPTION_TEXT(element).X,
 				(int) DESCRIPTION_TEXT(element).Y,
 				(int) DESCRIPTION_TEXT(element).Direction,
 				(int) DESCRIPTION_TEXT(element).Scale,
-					(int) DESCRIPTION_TEXT(element).Flags);
+				(int) DESCRIPTION_TEXT(element).Flags);
 
 		PIN_LOOP(element,
 			fprintf(FP, "\tPin(%i %i %i %i ",
 				(int) pin->X, (int) pin->Y,
 				(int) pin->Thickness, (int) pin->DrillingHole);
 			PrintQuotedString(FP, EMPTY(pin->Name));
-			fprintf(FP, " 0x%04x)\n", (int) pin->Flags);
+			fprintf(FP, " 0x%08x)\n", (int) pin->Flags);
 		);
 		PAD_LOOP(element,
 			fprintf(FP, "\tPad(%i %i %i %i %i ",
@@ -364,7 +364,7 @@ static void WritePCBElementData(FILE *FP)
 				(int) pad->Point2.X, (int) pad->Point2.Y,
 				(int) pad->Thickness);
 			PrintQuotedString(FP, EMPTY(pad->Name));
-			fprintf(FP, " 0x%04x)\n", (int) pad->Flags);
+			fprintf(FP, " 0x%08x)\n", (int) pad->Flags);
 		);
 
 		ELEMENTLINE_LOOP(element,
@@ -395,7 +395,7 @@ static void WritePCBLayerData(FILE *FP, Cardinal Number)
 	LayerTypePtr	layer = &PCB->Data->Layer[Number];
 
 		/* write information about non empty layers */
-	if (layer->LineN || layer->TextN || layer->PolygonN || 
+	if (layer->LineN || layer->ArcN || layer->TextN || layer->PolygonN || 
 		(layer->Name && *layer->Name))
 	{
 		fprintf(FP, "Layer(%i ", (int) Number+1);
@@ -403,23 +403,29 @@ static void WritePCBLayerData(FILE *FP, Cardinal Number)
 		fputs(")\n(\n", FP);
 
 		LINE_LOOP(layer,
-			fprintf(FP, "\tLine(%i %i %i %i %i 0x%04x)\n",
+			fprintf(FP, "\tLine(%i %i %i %i %i 0x%08x)\n",
 				(int) line->Point1.X, (int) line->Point1.Y,
 				(int) line->Point2.X, (int) line->Point2.Y,
 				(int) line->Thickness, (int) line->Flags);
 		);
+		ARC_LOOP(layer,
+			fprintf(FP, "\tArc(%i %i %i %i %i %i %i 0x%08x)\n",
+				(int) arc->X, (int) arc->Y, (int) arc->Width,
+				(int) arc->Height, (int) arc->Thickness,
+				arc->StartAngle, arc->Delta, (int) arc->Flags);
+		);
 		TEXT_LOOP(layer,
 			fprintf(FP, "\tText(%i %i %i %i ",
 				(int) text->X, (int) text->Y,
 				(int) text->Direction, (int) text->Scale);
 			PrintQuotedString(FP, EMPTY(text->TextString));
-			fprintf(FP, " 0x%04x)\n", (int) text->Flags);
+			fprintf(FP, " 0x%08x)\n", (int) text->Flags);
 		);
 		POLYGON_LOOP(layer,
 			{
 				int	i = 0;
 
-				fprintf(FP, "\tPolygon(0x%04x)\n\t(", polygon->Flags);
+				fprintf(FP, "\tPolygon(0x%08x)\n\t(", polygon->Flags);
 				POLYGONPOINT_LOOP(polygon,
 					{
 						if (i++ % 5 == 0)
@@ -478,9 +484,9 @@ static int WritePCBFile(char *Filename)
  */
 static int WritePCBPipe(char *Filename)
 {
-			FILE				*fp;
-			int					result;
-			char				*p;
+	FILE				*fp;
+	int				result;
+	char				*p;
 	static  DynamicStringType	command;
 
 		/* setup commandline */
@@ -594,11 +600,11 @@ void RemoveTMPData(void)
  */
 int ReadLibraryContents(void)
 {
-	static	char				*command = NULL;
-			char				inputline[MAX_LIBRARY_LINE_LENGTH+1];
-			FILE				*resultFP;
-			LibraryMenuTypePtr	menu = Library.Menu;
-			LibraryEntryTypePtr	entry;
+	static	char		*command = NULL;
+	char			inputline[MAX_LIBRARY_LINE_LENGTH+1];
+	FILE			*resultFP;
+	LibraryMenuTypePtr	menu = Library.Menu;
+	LibraryEntryTypePtr	entry;
 
 		/* release old command string */
 	MyFree(&command);
