@@ -22,7 +22,7 @@
  *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
  *  Thomas.Nau@rz.uni-ulm.de
  *
- *  RCS: $Id: insert.h,v 143.1 1996/09/16 09:08:41 nau Exp $
+ *  RCS: $Id: insert.h,v 1.1 1998/02/19 18:06:34 sync Exp $
  */
 
 /* prototypes for inserting points into objects
@@ -33,11 +33,11 @@
 
 #include "global.h"
 
-#define	INSERT_TYPES	(POLYGON_TYPE | LINE_TYPE)
+#define	INSERT_TYPES	(POLYGON_TYPE | LINE_TYPE | RATLINE_TYPE)
 
 /* ---------------------------------------------------------------------------
  * prototypes
  */
-void	*InsertPointIntoObject(int, void *, void *, void *, Position, Position);
+void	*InsertPointIntoObject(int, void *, void *, Cardinal *, Position, Position);
 
 #endif
