@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: main.c,v 143.1 1996/09/16 09:08:44 nau Exp $";
+static	char	*rcsid = "$Id: main.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 
 /* main program, initializes some stuff and handles user input
  */
@@ -54,6 +54,7 @@ static	char	*rcsid = "$Id: main.c,v 143.1 1996/09/16 09:08:44 nau Exp $";
 
 #include <X11/cursorfont.h> 
 #include <X11/Shell.h> 
+#include <X11/Xutil.h>
 #include <X11/Xatom.h> 
 #include <X11/Xaw/Command.h> 
 #include <X11/Xaw/Form.h> 
@@ -85,6 +86,23 @@ static	void	InitHandler(void);
 static	void	InitWidgets(void);
 static	void	InitDeviceDriver(void);
 
+/* ---------------------------------------------------------------------------
+ *  Stipple patterns with 50 % pixels
+ */
+#define gray_size 8
+typedef char	pattern[8];
+static pattern gray_bits[] ={ 
+	{0xa5, 0x5a, 0xa5, 0x5a, 0xa5, 0x5a, 0xa5, 0x5a},
+	{0x33, 0xcc, 0x33, 0xcc, 0x33, 0xcc, 0x33, 0xcc},
+	{0xcc, 0x33, 0xcc, 0x33, 0xcc, 0x33, 0xcc, 0x33},
+	{0x88, 0x77, 0x88, 0x77, 0x88, 0x77, 0x88, 0x77},
+	{0x77, 0x88, 0x77, 0x88, 0x77, 0x88, 0x77, 0x88},
+	{0xee, 0x11, 0xee, 0x11, 0xee, 0x11, 0xee, 0x11},
+	{0x11, 0xee, 0x11, 0xee, 0x11, 0xee, 0x11, 0xee},
+	{0x96, 0x69, 0x96, 0x69, 0x96, 0x69, 0x96, 0x69},
+	{0x69, 0x96, 0x69, 0x96, 0x69, 0x96, 0x69, 0x96}
+};
+
 /* ---------------------------------------------------------------------------
  * default translations
  */
@@ -116,12 +134,18 @@ static	String			Fallback[] = {
  * the number of layers has to be adapted here and in 'global.h'
  */
 static	XtResource		ToplevelResources[] = {
+	{ "rubberBandMode", "RubberBandMode", XtRBoolean, sizeof(Boolean),
+	  XtOffsetOf(SettingType, RubberBandMode), XtRString, "True" },
 	{ "absoluteGrid", "AbsoluteGrid", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, AbsoluteGrid), XtRString, "True" },
 	{ "alignmentDistance", "AlignmentDistance", XtRDimension, sizeof(Dimension),
 	  XtOffsetOf(SettingType, AlignmentDistance), XtRString, "200" },
 	{ "allDirectionLines", "AllDirectionLines", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, AllDirectionLines), XtRString, "True" },
+	{ "apertureCommand", "ApertureCommand", XtRString, sizeof(String),
+	  XtOffsetOf(SettingType, ApertureCommand), XtRString, "cat %f" },
+	{ "aperturePath", "AperturePath", XtRString, sizeof(String),
+	  XtOffsetOf(SettingType, AperturePath), XtRString, PCBLIBDIR },
 	{ "backupInterval", "BackupInterval", XtRInt, sizeof(long),
 	  XtOffsetOf(SettingType, BackupInterval), XtRString, "300" },
 	{ "charactersPerLine", "CharactersPerLine", XtRInt, sizeof(int),
@@ -154,9 +178,6 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, Grid), XtRString, "100" },
 	{ "gridColor", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, GridColor), XtRString, XtDefaultForeground },
-	{ "groundplaneColor", XtCColor, XtRPixel, sizeof(Pixel),
-	  XtOffsetOf(SettingType, GroundplaneColor),
-	  XtRString, XtDefaultForeground },
 	{ "invisibleObjectsColor", XtCColor, XtRPixel, sizeof(Pixel),
 	  XtOffsetOf(SettingType, InvisibleObjectsColor),
 	  XtRString, XtDefaultForeground },
@@ -270,10 +291,16 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, PrintFile), XtRString, "%f.output" },
 	{ "raiseLogWindow", "RaiseLogWindow", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, RaiseLogWindow), XtRString, "True" },
+	{ "ratCommand", "RatCommand", XtRString, sizeof(String),
+	  XtOffsetOf(SettingType, RatCommand), XtRString, "cat %f" },
+	{ "ratPath", "RatPath", XtRString, sizeof(String),
+	  XtOffsetOf(SettingType, RatPath), XtRString, "." },
 	{ "resetAfterElement", "ResetAfterElement", XtRBoolean, sizeof(Boolean),
 	  XtOffsetOf(SettingType, ResetAfterElement), XtRString, "False" },
 	{ "ringBellWhenFinished", "RingBellWhenFinished",XtRBoolean,sizeof(Boolean),
 	  XtOffsetOf(SettingType, RingBellWhenFinished), XtRString, "True" },
+	{ "rubberBandMode", "RubberBandMode", XtRBoolean, sizeof(Boolean),
+	  XtOffsetOf(SettingType, RubberBandMode), XtRString, "True" },
 	{ "saveCommand", "SaveCommand", XtRString, sizeof(String),
 	  XtOffsetOf(SettingType, SaveCommand), XtRString, "cat - > %f" },
 	{ "saveInTMP", "SaveInTMP", XtRBoolean, sizeof(Boolean),
@@ -282,6 +309,8 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, SaveLastCommand), XtRString, "False" },
 	{ "size", "Size", XtRString, sizeof(String),
 	  XtOffsetOf(SettingType, Size), XtRString, DEFAULT_SIZE },
+	{ "stipplePolygons", "StipplePolygons", XtRBoolean, sizeof(Boolean),
+	  XtOffsetOf(SettingType, StipplePolygons), XtRString, "False" },
 	{ "textScale", "TextScale", XtRDimension, sizeof(Dimension),
 	  XtOffsetOf(SettingType, TextScale), XtRString, "100" },
 	{ "useLogWindow", "UseLogWindow", XtRBoolean, sizeof(Boolean),
@@ -301,6 +330,8 @@ static	XtResource		ToplevelResources[] = {
 	  XtOffsetOf(SettingType, ViaThickness), XtRString, "40" },
 	{ "volume", "Volume", XtRInt, sizeof(int),
 	  XtOffsetOf(SettingType, Volume), XtRString, "100" },
+	{ "warnColor", XtCColor, XtRPixel, sizeof(Pixel),
+	  XtOffsetOf(SettingType, WarnColor), XtRString, XtDefaultForeground },
 	{ "zoom", "Zoom", XtRInt, sizeof(int),
 	  XtOffsetOf(SettingType, Zoom), XtRString, "3" }};
 
@@ -310,9 +341,12 @@ static	XtResource		ToplevelResources[] = {
 static	XrmOptionDescRec	CommandLineOptions[] = {
 	{ "-alldirections","allDirectionLines", XrmoptionNoArg, (caddr_t) "True" },
 	{ "+alldirections","allDirectionLines", XrmoptionNoArg, (caddr_t) "False" },
+	{ "-rubberband","rubberBandMode", XrmoptionNoArg, (caddr_t) "False" },
+	{ "+rubberband","rubberBandMode", XrmoptionNoArg, (caddr_t) "True" },
 	{ "-backup", "backupInterval", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-c", "charactersPerLine", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-fontfile", "fontFile", XrmoptionSepArg, (caddr_t) NULL },
+	{ "-laperture", "apertureCommand", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-lelement", "elementCommand", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-lfile", "fileCommand", XrmoptionSepArg, (caddr_t) NULL },
 	{ "-lfont", "fontCommand", XrmoptionSepArg, (caddr_t) NULL },
@@ -341,18 +375,26 @@ static	XrmOptionDescRec	CommandLineOptions[] = {
  */
 static	XtActionsRec	Actions[] = {
 	{ "MovePointer", ActionMovePointer },
+	{ "ToggleHideName", ActionToggleHideName },
+	{ "ChangeHole", ActionChangeHole },
 	{ "SetValue", ActionSetValue },
 	{ "FinishInputDialog", ActionFinishInputDialog },
 	{ "Quit", ActionQuit },
 	{ "Connection", ActionConnection },
 	{ "Command", ActionCommand },
 	{ "Display", ActionDisplay },
+	{ "Report", ActionReport },
 	{ "Mode", ActionMode },
+	{ "MoveToSilk", ActionMoveToSilk },
 	{ "RemoveSelected", ActionRemoveSelected },
+	{ "DeleteRats", ActionDeleteRats },
+	{ "AddRats", ActionAddRats },
+	{ "MarkCrosshair", ActionMarkCrosshair },
 	{ "ChangeSize", ActionChangeSize },
-	{ "Change2ndSize", ActionChange2ndSize },
+	{ "ChangeDrillSize", ActionChange2ndSize },
 	{ "ChangeName", ActionChangeName },
 	{ "ChangeSquare", ActionChangeSquare },
+	{ "ChangeOctagon", ActionChangeOctagon },
 	{ "Select", ActionSelect },
 	{ "Unselect", ActionUnselect },
 	{ "Save", ActionSave },
@@ -363,9 +405,9 @@ static	XtActionsRec	Actions[] = {
 	{ "Bell", ActionBell },
 	{ "PasteBuffer", ActionPasteBuffer },
 	{ "Undo", ActionUndo },
+	{ "Redo", ActionRedo },
 	{ "Polygon", ActionPolygon },
 	{ "EditLayerGroups", ActionEditLayerGroups },
-	{ "Groundplane", ActionGroundplane },
 	{ "MoveToCurrentLayer", ActionMoveToCurrentLayer },
 	{ "SwitchDrawingLayer", ActionSwitchDrawingLayer }};
 
@@ -380,12 +422,12 @@ static	XtActionsRec	Actions[] = {
  */
 static void GetSizeOfDrawingArea(void)
 {
-	Dimension		width, height,
-					scroll_width, scroll_height,
-					border_left, border_bottom;
-	Position		x, y,
-					maxw = TO_SCREEN(PCB->MaxWidth),
-					maxh = TO_SCREEN(PCB->MaxHeight);
+	Dimension	width, height,
+			scroll_width, scroll_height,
+			border_left, border_bottom;
+	Position	x, y,
+			maxw = TO_SCREEN(PCB->MaxWidth),
+			maxh = TO_SCREEN(PCB->MaxHeight);
 
 	XtVaGetValues(Output.Viewport,
 		XtNwidth, &width,
@@ -494,13 +536,26 @@ static void CB_Backup(XtPointer ClientData, XtIntervalId *ID)
 static void OutputEvent(Widget W,
 	XtPointer ClientData, XEvent *Event, Boolean *Flag)
 {
-			XExposeEvent	*event = (XExposeEvent *) Event;
-	static	Boolean			has_entered = False;
+	int			ksym;
+	XExposeEvent		*event = (XExposeEvent *) Event;
+	static	Boolean		has_entered = False;
 
 	switch(Event->type)
 	{
-		case Expose:				/* expose event means redraw */
-			if (VALID_PIXMAP(PCB->SaveUnder))
+		case Expose:	/* expose event means redraw */
+			if (event->send_event)
+			{
+				XSetRegion(Dpy, Output.fgGC, UpRegion);
+				XSetRegion(Dpy, Output.bgGC, UpRegion);
+				XSetRegion(Dpy, Output.GridGC, UpRegion);
+				RedrawOutput();
+				XDestroyRegion(UpRegion);
+				UpRegion = XCreateRegion();
+				XSetRegion(Dpy, Output.fgGC, FullRegion);
+				XSetRegion(Dpy, Output.bgGC, FullRegion);
+				XSetRegion(Dpy, Output.GridGC, FullRegion);
+			}
+			else if (VALID_PIXMAP(PCB->SaveUnder))
 			{
 					/* make sure that cursor is in the same state
 					 * as when a dialog poped up. The state is determined
@@ -513,7 +568,7 @@ static void OutputEvent(Widget W,
 					event->width, event->height,
 					event->x, event->y))
 				{
-					ClearAndRedrawOutput();
+					RedrawOutput();
 				}
 				RestoreCrosshair(False);
 			}
@@ -547,6 +602,17 @@ static void OutputEvent(Widget W,
 		case MotionNotify:
 			EventMoveCrosshair((XMotionEvent *) Event);
 			break;
+
+		case KeyPress:
+		case KeyRelease:
+			ksym = XKeycodeToKeysym(Dpy, ((XKeyEvent *) Event)->keycode, 0);
+			if (ksym == XK_Shift_R || ksym == XK_Shift_L)
+			{
+				HideCrosshair(False);
+				AdjustAttachedObjects();
+				RestoreCrosshair(True);
+			}
+			break;
 	}
 } 
 
@@ -555,8 +621,9 @@ static void OutputEvent(Widget W,
  */
 static void InitShell(int *Argc, char **Argv)
 {
-	int				i, x, y;
+	int		i, x, y;
 	unsigned int	width, height;
+	XModifierKeymap		*modMap;
 
 		/* init application toplevel window, get resources */
 	Output.Toplevel = XtAppInitialize(&Context, "Pcb",
@@ -591,6 +658,9 @@ static void InitShell(int *Argc, char **Argv)
 	Settings.MaxHeight = MIN(MAX_COORD, MAX((Dimension) height, MIN_SIZE));
 	Settings.Volume = MIN(100, MAX(-100, Settings.Volume));
 	ParseGroupString(Settings.Groups, &Settings.LayerGroups);
+	modMap = XGetModifierMapping(Dpy);
+	ShiftKeyIndex = modMap->modifiermap[ShiftMapIndex] / 8;
+	ShiftKeyMask = 1 << (modMap->modifiermap[ShiftMapIndex] % 8);
 }
 
 /* ---------------------------------------------------------------------------
@@ -628,7 +698,7 @@ static void InitStatusLine(Widget Parent, Widget Top, Widget Left)
  */
 static void InitViewport(Widget Parent, Widget Top, Widget Left)
 {
-		/* viewport widget to to handle scrolling */
+		/* viewport widget to handle scrolling */
 	Output.Viewport = XtVaCreateManagedWidget("viewport", viewportWidgetClass,
 		Parent,
 		XtNfromHoriz, Left,
@@ -651,7 +721,7 @@ static void InitViewport(Widget Parent, Widget Top, Widget Left)
 		/* add event handlers */
 	XtAddEventHandler(Output.Output,
 		ExposureMask | LeaveWindowMask | EnterWindowMask |
-		StructureNotifyMask | PointerMotionMask,
+		StructureNotifyMask | PointerMotionMask | KeyPressMask | KeyReleaseMask,
 		False, (XtEventHandler) OutputEvent, NULL);
 	XtAddEventHandler(Output.Viewport,
 		StructureNotifyMask | VisibilityChangeMask,
@@ -697,6 +767,8 @@ static void InitIcon(void)
  */
 static void InitGC(void)
 {
+	Cardinal	i;
+
 	XtVaGetValues(Output.Output, XtNbackground, &Settings.bgColor, NULL);
 	Output.fgGC = XCreateGC(Dpy, Output.OutputWindow, 0, NULL);
 	Output.bgGC = XCreateGC(Dpy, Output.OutputWindow, 0, NULL);
@@ -708,8 +780,15 @@ static void InitGC(void)
 
 	XSetForeground(Dpy, Output.bgGC, Settings.bgColor);
 	XSetFillRule(Dpy, Output.fgGC, WindingRule);
+	XSetFillStyle(Dpy, Output.fgGC, FillSolid);
+	for (i =0; i < 9; i++)
+	{
+		Stipples[i] = XCreateBitmapFromData(Dpy, Output.OutputWindow, 
+		gray_bits[i], gray_size, gray_size);
+	}
 	XSetState(Dpy, Output.GridGC, Settings.GridColor, Settings.bgColor,
 		GXinvert, AllPlanes);
+	XSetLineAttributes(Dpy, Output.GridGC, 2, LineSolid, CapButt, JoinMiter);
 
 		/* dont create graphic expose events from XCopyArea() */
 	XSetGraphicsExposures(Dpy, Output.fgGC, False);
@@ -720,7 +799,7 @@ static void InitGC(void)
  */
 static void InitHandler(void)
 {
-		/* install a new error handler and catch terminating signals */
+	/* install a new error handler and catch terminating signals */
 	XtAppSetErrorHandler(Context, X11ErrorHandler);
 
 /*
@@ -754,7 +833,7 @@ static void InitWidgets(void)
 	InitControlPanel(Output.MasterForm, Output.Menu, NULL);
 	InitCursorPosition(Output.MasterForm, NULL, Output.Menu);
 	InitViewport(Output.MasterForm, Output.Menu, Output.Control);
-	InitStatusLine(Output.MasterForm, Output.Viewport, Output.Control);
+	InitStatusLine(Output.MasterForm, Output.Viewport, NULL);
 
 		/* set actions and install translations;
 		 * the menu must be initialized before the actions are added
@@ -771,14 +850,18 @@ static void InitWidgets(void)
 		 * resizable even if a geometry is passed to the application
 		 */
 	XtRealizeWidget(Output.Toplevel);
+/* the Mode buttons need the Toplevel widget to be realized before
+ * they can be initialized
+ */
+	InitModeButtons(Output.MasterForm, Output.Control, NULL);
 	XtDestroyWidget(Output.StatusLine);
-	InitStatusLine(Output.MasterForm, Output.Viewport, Output.Control);
+	InitStatusLine(Output.MasterForm, Output.Viewport, NULL);
 	Output.OutputWindow = XtWindow(Output.Output);	
 
 	InitAccelerators();
 	InitGC();
-	InitIcon();
 	InitCrosshair();
+	InitIcon();
 	InitHandler();
 	InitBuffers();
 	SetOutputXCursor(XC_crosshair);
@@ -793,10 +876,13 @@ static void InitWidgets(void)
  */
 static void InitDeviceDriver(void)
 {
-	int	i;
+	DeviceInfoTypePtr	ptr = PrintingDevice;
 
-	for (i = 0; i < ENTRIES(PrintingDevice); i++)
-		PrintingDevice[i].Info = PrintingDevice[i].Query();
+	while (ptr->Query)
+	{
+		ptr->Info = ptr->Query();
+		ptr++;
+	}
 }
 
 /* ---------------------------------------------------------------------- 
@@ -804,6 +890,9 @@ static void InitDeviceDriver(void)
  */
 int main(int argc, char *argv[]) 
 {
+	Cardinal	i;
+	XRectangle	Big;
+
 		/* init application:
 		 * - make program name available for error handlers
 		 * - evaluate special options
@@ -843,6 +932,15 @@ int main(int argc, char *argv[])
 		/* update zoom and grid... */
 	UpdateSettingsOnScreen();
 	GetSizeOfDrawingArea();
+		/* set up the Erasure client message */
+	Erasure.type = Expose;
+	Erasure.display = Dpy;
+	Erasure.window = Output.OutputWindow;
+	UpRegion = XCreateRegion();
+	FullRegion = XCreateRegion();
+	Big.x = Big.y = 0;
+	Big.width = Big.height = MAX_COORD;
+	XUnionRectWithRegion(&Big, FullRegion, FullRegion);
 	switch(--argc)
 	{
 		case 0:		/* only program name */
@@ -882,12 +980,9 @@ int main(int argc, char *argv[])
 #endif
 
 		/* popup main window and initialize error log window
-		 * the mode buttons require a 'drawable' so the widgets have to be
-		 * realized before calling the init routine.
 		 * If the window isn't mapped the statusline won't be resizable
 		 * (dont know why)
 		 */
-	InitModeButtons(Output.MasterForm, Output.Control, NULL);
 	XMapWindow(Dpy, XtWindow(Output.Toplevel));
 	InitErrorLog();
 
@@ -899,5 +994,7 @@ int main(int argc, char *argv[])
 	XtAppAddTimeOut(Context, 1000*Settings.BackupInterval, CB_Backup, NULL);
 
 	XtAppMainLoop(Context);
+	for (i=0; i < 9; i++)
+		XFreePixmap(Dpy, Stipples[i]);
 	return(0);
 } 
