@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: menu.c,v 143.1 1996/09/16 09:08:44 nau Exp $";
+static	char	*rcsid = "$Id: menu.c,v 1.1 1998/02/19 18:06:34 sync Exp $";
 
 /* initializes menus and handles callbacks
  */
@@ -123,13 +123,16 @@ static	PopupEntryType	DisplayMenuEntries[] = {
 	{ "toggleGrid", "use absolute grid", CB_Position,
 		"Display,ToggleGrid", NULL },
 	{ "toggleAllDirections", "'all-direction' lines", CB_Action,
-		"Display,ToggleAllDirections", NULL },
+		"Display,Toggle45Degree", NULL },
+	{ "toggleRubberBandMode", "rubber band mode", CB_Action,
+		"Display,ToggleRubberBandMode", NULL },
 	{ "solderSide", "look at solder-side", CB_Action, "SwapSides", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "grid setting", NULL, NULL, NULL },
 	{ "grid05", " 5 mil", CB_Position, "SetValue,Grid,5", NULL },
 	{ "grid10", "10 mil", CB_Position, "SetValue,Grid,10", NULL },
 	{ "grid20", "20 mil", CB_Position, "SetValue,Grid,20", NULL },
+	{ "grid25", "25 mil", CB_Position, "SetValue,Grid,25", NULL },
 	{ "grid50", "50 mil", CB_Position, "SetValue,Grid,50", NULL },
 	{ "grid100", "100 mil", CB_Position, "SetValue,Grid,100", NULL },
 	{ "gridInc", "increment by 5", CB_Action, "SetValue,Grid,+5", NULL },
@@ -197,6 +200,8 @@ static	PopupEntryType	SelectionMenuEntries[] = {
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "remove", "remove selected objects", CB_Action,
 		"RemoveSelected", NULL },
+	{ "movetosilk", "selected text -> silkscreen", CB_Action,
+		"MoveToSilk,SelectedTexts", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "change size of selected objects", NULL, NULL, NULL },
 	{ "decrementline", "lines -10 mil", CB_Action,
@@ -222,13 +227,13 @@ static	PopupEntryType	SelectionMenuEntries[] = {
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "change drilling hole of selected objects", NULL, NULL, NULL },
 	{ "decrementviahole", "vias -10 mil", CB_Action,
-		"Change2ndSize,SelectedVias,-10", NULL },
+		"ChangeDrillSize,SelectedVias,-10", NULL },
 	{ "incrementviahole", "vias +10 mil", CB_Action,
-		"Change2ndSize,SelectedVias,+10", NULL },
+		"ChangeDrillSize,SelectedVias,+10", NULL },
 	{ "decrementpinhole", "pins -10 mil", CB_Action,
-		"Change2ndSize,SelectedPins,-10", NULL },
+		"ChangeDrillSize,SelectedPins,-10", NULL },
 	{ "incrementpinhole", "pins +10 mil", CB_Action,
-		"Change2ndSize,SelectedPins,+10", NULL },
+		"ChangeDrillSize,SelectedPins,+10", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "change square-flag of selected objects", NULL, NULL, NULL },
 	{ "elementsquare", "elements", CB_Action,
@@ -264,6 +269,7 @@ static	PopupEntryType	BufferMenuEntries[] = {
 		NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "clear", "clear buffer", CB_Action, "PasteBuffer,Clear", NULL },
+	{ "convert", "convert buffer to element", CB_Action, "PasteBuffer,Convert", NULL },
 	{ "line", NULL, NULL, NULL, NULL },
 	{ "header", "select current buffer", NULL, NULL, NULL },
 	{ "buffer1", "#1", CB_Action, "PasteBuffer,1", NULL },
@@ -303,29 +309,27 @@ static	MenuButtonType	ConnectionMenuButton =
  */
 static	PopupEntryType	UndoMenuEntries[] = {
 	{ "undo", "undo last destructive operation", CB_Action, "Undo", NULL },
+	{ "redo", "redo last undone operation", CB_Action, "Redo", NULL },
 	{ "clear", "clear undo-buffer", CB_Action, "Undo,ClearList", NULL },
 	{ NULL, NULL, NULL, NULL, NULL }};
 static	PopupMenuType	UndoMenu =
 	{ "undo",  NULL, UndoMenuEntries, NULL, NULL, NULL };
 static	MenuButtonType	UndoMenuButton =
 	{ "undo", "Undo", &UndoMenu, NULL};
+static	MenuButtonType	RedoMenuButon =
+	{ "redo", "Redo", &UndoMenu, NULL};
 
 /* ---------------------------------------------------------------------------
- * groundplane menu button
+ * report menu button
  */
-static	PopupEntryType	GroundplaneMenuEntries[] = {
-	{ "add", "add selected", CB_Action, "Groundplane,AddSelected", NULL },
-	{ "remove", "remove selected", CB_Action,
-		"Groundplane,RemoveSelected", NULL },
-	{ "clear", "clear list", CB_Action, "Groundplane,ClearList", NULL },
-	{ "line", NULL, NULL, NULL, NULL },
-	{ "select", "select members", CB_Action,
-		"Groundplane,SelectMembers", NULL },
+static	PopupEntryType	ReportMenuEntries[] = {
+	{ "report", "generate object report", CB_Position, "Report,Object", NULL },
+	{ "drillreport", "generate drill summary", CB_Action, "Report,DrillReport", NULL },
 	{ NULL, NULL, NULL, NULL, NULL }};
-static	PopupMenuType	GroundplaneMenu =
-	{ "groundplane",  NULL, GroundplaneMenuEntries, NULL, NULL, NULL };
-static	MenuButtonType	GroundplaneMenuButton =
-	{ "groundplane", "Groundplane", &GroundplaneMenu, NULL};
+static	PopupMenuType	ReportMenu =
+	{ "report",  NULL, ReportMenuEntries, NULL, NULL, NULL };
+static	MenuButtonType	ReportMenuButton =
+	{ "report", "Report", &ReportMenu, NULL};
 
 /* ----------------------------------------------------------------------
  * command buttons
@@ -406,7 +410,11 @@ static void CB_Action(Widget W, XtPointer ClientData, XtPointer CallData)
  */
 static void CB_Position(Widget W, XtPointer ClientData, XtPointer CallData)
 {
-	if (GetPosition("move pointer to the appropriate screen position and press a button"))
+	char *asString = (char *) ClientData;
+
+	if (!TEST_FLAG(ABSOLUTEFLAG, PCB) && strncmp("Display,ToggleGrid", asString, 18) == 0)
+		CB_Action(W, ClientData,CallData);
+	else if (GetPosition("move pointer to the appropriate screen position and press a button"))
 		CB_Action(W, ClientData, CallData);
 }
 
@@ -417,7 +425,7 @@ static void CB_Position(Widget W, XtPointer ClientData, XtPointer CallData)
 static void CB_ElementPosition(Widget W,
 	XtPointer ClientData, XtPointer CallData)
 {
-	if (GetPosition("press a button at the elements location"))
+	if (GetPosition("press a button at the element's location"))
 		CB_Action(W, ClientData, CallData);
 }
 
@@ -451,11 +459,13 @@ static void CBPOPUP_Display(Widget W, XtPointer ClientData, XtPointer CallData)
 {
 	RemoveCheckFromMenu(&DisplayMenu);
 	XtSetSensitive(XtNameToWidget(DisplayMenu.W, "displayGrid"),
-		(TO_SCREEN(PCB->Grid) >= MIN_GRID_DISTANCE));
+		GetGridFactor() != 0);
 	if (TEST_FLAG(ABSOLUTEFLAG, PCB))
 		CheckEntry(&DisplayMenu, "toggleGrid");
-	if (TEST_FLAG(ALLDIRCETIONFLAG, PCB))
+	if (TEST_FLAG(ALLDIRECTIONFLAG, PCB))
 		CheckEntry(&DisplayMenu, "toggleAllDirections");
+	if (TEST_FLAG(RUBBERBANDFLAG, PCB))
+		CheckEntry(&DisplayMenu, "toggleRubberBandMode");
 	if (Settings.DrawGrid)
 		CheckEntry(&DisplayMenu, "displayGrid");
 	if (Settings.ShowSolderSide)
@@ -663,7 +673,7 @@ void InitMenu(Widget Parent, Widget Top, Widget Left)
 	last = InitMenuButton(Parent, &BufferMenuButton, Top, last);
 	last = InitMenuButton(Parent, &ConnectionMenuButton, Top, last);
 	last = InitMenuButton(Parent, &UndoMenuButton, Top, last);
-	last = InitMenuButton(Parent, &GroundplaneMenuButton, Top, last);
+	last = InitMenuButton(Parent, &ReportMenuButton, Top, last);
 	Output.Menu = last;
 }
 
