@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: misc.c,v 143.1 1996/09/16 09:08:46 nau Exp $";
+static	char	*rcsid = "$Id: misc.c,v 1.2 1998/02/21 10:27:14 cad Exp $";
 
 /* misc functions used by several modules
  */
@@ -55,12 +55,14 @@ static	char	*rcsid = "$Id: misc.c,v 143.1 1996/09/16 09:08:46 nau Exp $";
 #include "remove.h"
 #include "rotate.h"
 #include "search.h"
+#include "set.h"
 
 #include <X11/cursorfont.h>
 #include <X11/Shell.h>
 #include <X11/Xaw/Scrollbar.h>
 #include <X11/Xaw/Simple.h>
 
+static void RightAngles( int, float *, float *);
 /* ---------------------------------------------------------------------------
  * prints copyright information
  */
@@ -95,9 +97,14 @@ void Usage(void)
 		"standard options are:\n"
 		"  -alldirections:           enable 'all-direction' lines\n"
 		"  +alldirections:           force 45 degree lines\n"
+		"  +rubberband:              enable rubberband move and rotate\n"
+		"  -rubberband:              turn off rubberband move and rotate\n"
 		"  -backup <seconds>:        time between two backups\n"
 		"  -c <number>:              characters per output-line\n"
 		"  -fontfile <file>:         read default font from this file\n"
+		"  -laperture <command>:     command to copy aperture files to stdout,\n"
+		"                            %%f is set to the filename\n"
+		"                            %%p is set to the seachpath\n"
 		"  -lelement <command>:      command to copy element files to stdout,\n"
 		"                            %%f is set to the filename\n"
 		"                            %%p is set to the seachpath\n"
@@ -146,7 +153,7 @@ void Usage(void)
 void SetPolygonBoundingBox(PolygonTypePtr Polygon)
 {
 	Position	minx, miny,
-				maxx, maxy;
+			maxx, maxy;
 
 	minx = miny = MAX_COORD;
 	maxx = maxy = 0;
@@ -165,18 +172,18 @@ void SetPolygonBoundingBox(PolygonTypePtr Polygon)
 /* ---------------------------------------------------------------------------
  * sets the bounding box of an elements
  */
-void SetElementBoundingBox(ElementTypePtr Element)
+void SetElementBoundingBox(ElementTypePtr Element, FontTypePtr Font)
 {
 	Position	minx, miny,
-				maxx, maxy;
+			maxx, maxy;
 	float		fminx, fminy,
-				fmaxx, fmaxy;
-	int			angle1,
-				angle2,
-				angle;
+			fmaxx, fmaxy;
+	int		angle1,
+			angle2,
+			angle;
 
 		/* first update the text objects */
-	ELEMENTTEXT_LOOP(Element, SetTextBoundingBox(&PCB->Font, text));
+	ELEMENTTEXT_LOOP(Element, SetTextBoundingBox(Font, text));
 
 		/* do not include the elementnames bounding box which
 		 * is handles seperatly
@@ -184,54 +191,54 @@ void SetElementBoundingBox(ElementTypePtr Element)
 	minx = miny = MAX_COORD;
 	maxx = maxy = 0;
 	ELEMENTLINE_LOOP(Element,
-		minx = MIN(minx, line->Point1.X);
-		miny = MIN(miny, line->Point1.Y);
-		minx = MIN(minx, line->Point2.X);
-		miny = MIN(miny, line->Point2.Y);
-		maxx = MAX(maxx, line->Point1.X);
-		maxy = MAX(maxy, line->Point1.Y);
-		maxx = MAX(maxx, line->Point2.X);
-		maxy = MAX(maxy, line->Point2.Y);
+		minx = MIN(minx, line->Point1.X - line->Thickness/2);
+		miny = MIN(miny, line->Point1.Y - line->Thickness/2);
+		minx = MIN(minx, line->Point2.X - line->Thickness/2);
+		miny = MIN(miny, line->Point2.Y - line->Thickness/2);
+		maxx = MAX(maxx, line->Point1.X + line->Thickness/2);
+		maxy = MAX(maxy, line->Point1.Y + line->Thickness/2);
+		maxx = MAX(maxx, line->Point2.X + line->Thickness/2);
+		maxy = MAX(maxy, line->Point2.Y + line->Thickness/2);
 	);
 	PIN_LOOP(Element,
-		minx = MIN(minx, pin->X);
-		miny = MIN(miny, pin->Y);
-		maxx = MAX(maxx, pin->X);
-		maxy = MAX(maxy, pin->Y);
+		minx = MIN(minx, pin->X - pin->Thickness/2);
+		miny = MIN(miny, pin->Y - pin->Thickness/2);
+		maxx = MAX(maxx, pin->X + pin->Thickness/2);
+		maxy = MAX(maxy, pin->Y + pin->Thickness/2);
 	);
 	PAD_LOOP(Element,
-		minx = MIN(minx, pad->Point1.X);
-		miny = MIN(miny, pad->Point1.Y);
-		minx = MIN(minx, pad->Point2.X);
-		miny = MIN(miny, pad->Point2.Y);
-		maxx = MAX(maxx, pad->Point1.X);
-		maxy = MAX(maxy, pad->Point1.Y);
-		maxx = MAX(maxx, pad->Point2.X);
-		maxy = MAX(maxy, pad->Point2.Y);
+		minx = MIN(minx, pad->Point1.X - pad->Thickness/2);
+		miny = MIN(miny, pad->Point1.Y - pad->Thickness/2);
+		minx = MIN(minx, pad->Point2.X - pad->Thickness/2);
+		miny = MIN(miny, pad->Point2.Y - pad->Thickness/2);
+		maxx = MAX(maxx, pad->Point1.X + pad->Thickness/2);
+		maxy = MAX(maxy, pad->Point1.Y + pad->Thickness/2);
+		maxx = MAX(maxx, pad->Point2.X + pad->Thickness/2);
+		maxy = MAX(maxy, pad->Point2.Y + pad->Thickness/2);
 	);
 	ARC_LOOP(Element,
-			/* arc->StartAngle is in [0,360], arc->Delta in [0,360] */
+		/* arc->StartAngle is in [0,360], arc->Delta in [0,360] */
 		angle1 = arc->StartAngle;
 		angle2 = arc->StartAngle +arc->Delta;
 
-			/* initialize limits */
-		fminx = MIN(cos(M180*(float) angle1),cos(M180*(float) angle2));
-		fmaxx = MAX(cos(M180*(float) angle1),cos(M180*(float) angle2));
+		/* initialize limits */
+		fminx = MIN(-cos(M180*(float) angle1),-cos(M180*(float) angle2));
+		fmaxx = MAX(-cos(M180*(float) angle1),-cos(M180*(float) angle2));
 		fminy = MIN(sin(M180*(float) angle1),sin(M180*(float) angle2));
 		fmaxy = MAX(sin(M180*(float) angle1),sin(M180*(float) angle2));
 
-			/* loop and check all angles n*180
-			 * with angle1 <= a <= angle2
-			 */
+		/* loop and check all angles n*180
+		 * with angle1 <= a <= angle2
+		 */
 		for (angle = (angle1 /180 +1)*180; angle < angle2; angle += 180)
 		{
-			fminx = MIN(cos(M180 *(float) angle), fminx);
-			fmaxx = MAX(cos(M180 *(float) angle), fmaxx);
+			fminx = MIN(-cos(M180 *(float) angle), fminx);
+			fmaxx = MAX(-cos(M180 *(float) angle), fmaxx);
 		}
 
-			/* loop and check all angles n*180+90
-			 * with angle1 <= a <= angle2
-			 */
+		/* loop and check all angles n*180+90
+		 * with angle1 <= a <= angle2
+		 */
 		for (angle = ((angle1+90)/180)*180+90; angle < angle2; angle += 180)
 		{
 			fminy = MIN(sin(M180 *(float) angle), fminy);
@@ -256,8 +263,8 @@ void SetTextBoundingBox(FontTypePtr FontPtr, TextTypePtr Text)
 {
 	SymbolTypePtr	symbol = FontPtr->Symbol;
 	unsigned char	*s = (unsigned char *) Text->TextString;
-	Position		width = 0,
-					height = 0;
+	Position	width = 0,
+			height = 0;
 
 		/* calculate size of the bounding box */
 	for (; s && *s; s++)
@@ -310,9 +317,10 @@ Boolean IsDataEmpty(DataTypePtr Data)
 	hasNoObjects &= (Data->ElementN == 0);
 	for (i = 0; i < MAX_LAYER; i++)
 		hasNoObjects = hasNoObjects &&
-			Data->Layer[i].LineN == 0 && 
-			Data->Layer[i].TextN == 0 &&
-			Data->Layer[i].PolygonN == 0;
+		Data->Layer[i].LineN == 0 && 
+		Data->Layer[i].ArcN == 0 &&
+		Data->Layer[i].TextN == 0 &&
+		Data->Layer[i].PolygonN == 0;
 	return(hasNoObjects);
 }
 
@@ -348,14 +356,20 @@ BoxTypePtr GetDataBoundingBox(DataTypePtr Data)
 		);
 	);
 	ALLLINE_LOOP(Data,
-		box.X1 = MIN(box.X1, line->Point1.X);
-		box.Y1 = MIN(box.Y1, line->Point1.Y);
-		box.X1 = MIN(box.X1, line->Point2.X);
-		box.Y1 = MIN(box.Y1, line->Point2.Y);
-		box.X2 = MAX(box.X2, line->Point1.X);
-		box.Y2 = MAX(box.Y2, line->Point1.Y);
-		box.X2 = MAX(box.X2, line->Point2.X);
-		box.Y2 = MAX(box.Y2, line->Point2.Y);
+		box.X1 = MIN(box.X1, line->Point1.X - line->Thickness/2);
+		box.Y1 = MIN(box.Y1, line->Point1.Y - line->Thickness/2);
+		box.X1 = MIN(box.X1, line->Point2.X - line->Thickness/2);
+		box.Y1 = MIN(box.Y1, line->Point2.Y - line->Thickness/2);
+		box.X2 = MAX(box.X2, line->Point1.X + line->Thickness/2);
+		box.Y2 = MAX(box.Y2, line->Point1.Y + line->Thickness/2);
+		box.X2 = MAX(box.X2, line->Point2.X + line->Thickness/2);
+		box.Y2 = MAX(box.Y2, line->Point2.Y + line->Thickness/2);
+	);
+	ALLARC_LOOP(Data,
+		box.X1 = MIN(box.X1, arc->BoundingBox.X1);
+		box.Y1 = MIN(box.Y1, arc->BoundingBox.Y1);
+		box.X2 = MAX(box.X2, arc->BoundingBox.X2);
+		box.Y2 = MAX(box.Y2, arc->BoundingBox.Y2);
 	);
 	ALLTEXT_LOOP(Data,
 		box.X1 = MIN(box.X1, text->BoundingBox.X1);
@@ -378,9 +392,9 @@ BoxTypePtr GetDataBoundingBox(DataTypePtr Data)
  */
 void CenterDisplay(Position X, Position Y)
 {
-	Position	old_x, old_y,			/* output window in viewport */
-				new_x, new_y;
-	float		top;					/* top of thumb */
+	Position	old_x, old_y,	/* output window in viewport */
+			new_x, new_y;
+	float		top;		/* top of thumb */
 
 		/* save old values (offset of upper/left corner) */
 	XtVaGetValues(Output.Output, XtNx, &old_x, XtNy, &old_y, NULL);
@@ -413,10 +427,10 @@ void CenterDisplay(Position X, Position Y)
  */
 void SetFontInfo(FontTypePtr Ptr)
 {
-	Cardinal		i, j;
+	Cardinal	i, j;
 	SymbolTypePtr	symbol;
-	LineTypePtr		line;
-	Position		totalminy = MAX_COORD;
+	LineTypePtr	line;
+	Position	totalminy = MAX_COORD;
 
 		/* calculate cell with and height (is at least DEFAULT_CELLSIZE)
 		 * maximum cell width and height
@@ -426,8 +440,8 @@ void SetFontInfo(FontTypePtr Ptr)
 	Ptr->MaxHeight = DEFAULT_CELLSIZE;
 	for (i = 0, symbol = Ptr->Symbol; i <= MAX_FONTPOSITION; i++, symbol++)
 	{
-		Position		minx, miny,
-						maxx, maxy;
+		Position	minx, miny,
+				maxx, maxy;
 
 			/* next one if the index isn't used or symbol is empty (SPACE) */
 		if (!symbol->Valid || !symbol->LineN)
@@ -453,7 +467,7 @@ void SetFontInfo(FontTypePtr Ptr)
 
 			/* set symbol bounding box with a minimum cell size of (1,1) */
 		symbol->Width = maxx -minx +1;
-		symbol->Height = maxy -miny +1;
+		symbol->Height = maxy +1;
 
 			/* check total min/max  */
 		Ptr->MaxWidth = MAX(Ptr->MaxWidth, symbol->Width);
@@ -464,8 +478,11 @@ void SetFontInfo(FontTypePtr Ptr)
 		/* move coordinate system to the upper edge (lowest y on screen) */
 	for (i = 0, symbol = Ptr->Symbol; i <= MAX_FONTPOSITION; i++, symbol++)
 		if (symbol->Valid)
+		{
+			symbol->Height -= totalminy;
 			for (line = symbol->Line, j = symbol->LineN; j; j--, line++)
 				MOVE_LINE_LOWLEVEL(line, 0, -totalminy);
+		}
 
 		/* setup the box for the default symbol */
 	Ptr->DefaultSymbol.X1 = Ptr->DefaultSymbol.Y1 = 0;
@@ -479,11 +496,11 @@ void SetFontInfo(FontTypePtr Ptr)
  */
 int ParseGroupString(char *s, LayerGroupTypePtr LayerGroup)
 {
-	int		group,
-			member,
-			layer;
+	int	group,
+		member,
+		layer;
 	Boolean	c_set = False,	/* flags for the two special layers to */
-			s_set = False;	/* provide a default setting for old formats */
+		s_set = False;	/* provide a default setting for old formats */
 
 		/* clear struct */
 	memset(LayerGroup, 0, sizeof (LayerGroupType));
@@ -558,7 +575,7 @@ void QuitApplication(void)
 {
 		/* save data if necessary */
 	if (PCB->Changed && Settings.SaveInTMP)
-			EmergencySave();
+		EmergencySave();
 	exit(0);
 }
 
@@ -571,7 +588,7 @@ char *EvaluateFilename(char *Template, char *Path, char *Filename,
 	char *Parameter)
 {
 	static	DynamicStringType	command;
-			char				*p;
+	char				*p;
 
 	DSClearString(&command);
 	for (p = Template; *p; p++)
@@ -598,9 +615,9 @@ char *EvaluateFilename(char *Template, char *Path, char *Filename,
 char *ExpandFilename(char *Dirname, char *Filename)
 {
 	static	DynamicStringType	answer;
-			char				*command;
-			FILE				*pipe;
-			int					c;
+	char				*command;
+	FILE				*pipe;
+	int				c;
 
 		/* allocate memory for commandline and build it */
 	DSClearString(&answer);
@@ -650,9 +667,20 @@ Boolean GetPosition(char *MessageText)
 	KeySym		keysym;
 	char		buffer[10];
 	Cursor		oldCursor;
+	int		oldObjState;
+	int		oldLineState;
+	int		oldBoxState;
 
 	MessagePrompt(MessageText);
+	oldObjState = Crosshair.AttachedObject.State;
+	oldLineState = Crosshair.AttachedLine.State;
+	oldBoxState = Crosshair.AttachedBox.State;
+	HideCrosshair(True);
+	Crosshair.AttachedObject.State = STATE_FIRST;
+	Crosshair.AttachedLine.State = STATE_FIRST;
+	Crosshair.AttachedBox.State = STATE_FIRST;
 	oldCursor = SetOutputXCursor(XC_hand2);
+	RestoreCrosshair(True);
 
 		/* eat up all events that would cause actions to be performed */
 	for(;;)
@@ -679,6 +707,11 @@ Boolean GetPosition(char *MessageText)
 						/* abort on any other key;
 						 * restore cursor and clear message line
 						 */
+					HideCrosshair(True);
+					Crosshair.AttachedObject.State = oldObjState;
+					Crosshair.AttachedLine.State = oldLineState;
+					Crosshair.AttachedBox.State = oldBoxState;
+					RestoreCrosshair(True);
 					SetOutputXCursor(oldCursor);
 					MessagePrompt(NULL);
 					return(any->type == ButtonPress);
@@ -720,7 +753,8 @@ void SaveOutputWindow(void)
 
 			/* make sure that cursor is off */
 		if (VALID_PIXMAP(PCB->SaveUnder))
-			XCopyArea(Dpy, Output.OutputWindow, PCB->SaveUnder, Output.fgGC,
+			XCopyArea(Dpy, Output.OutputWindow,
+				PCB->SaveUnder, Output.fgGC,
 				Output.OffsetX, Output.OffsetY,
 				Output.Width, Output.Height,
 				0, 0);
@@ -734,7 +768,7 @@ void SaveOutputWindow(void)
 unsigned int SetOutputXCursor(unsigned int Shape)
 {
 	unsigned int	oldShape = Output.XCursorShape;
-	Cursor			oldCursor = Output.XCursor;
+	Cursor		oldCursor = Output.XCursor;
 
 		/* check if window exists to prevent from fatal errors */
 	if (Output.OutputWindow)
@@ -757,7 +791,7 @@ unsigned int SetOutputXCursor(unsigned int Shape)
  */
 int GetLayerNumber(DataTypePtr Data, LayerTypePtr Layer)
 {
-	int		i;
+	int	i;
 
 	for (i = 0; i < MAX_LAYER; i++)
 		if (Layer == &Data->Layer[i])
@@ -778,10 +812,10 @@ int GetLayerGroupNumberByPointer(LayerTypePtr Layer)
  */
 int GetLayerGroupNumberByNumber(Cardinal Layer)
 {
-	int		group,
-			entry;
+	int	group,
+		entry;
 
-	for (group = 0; group < MAX_LAYER+2; group++)
+	for (group = 0; group < MAX_LAYER; group++)
 		for (entry = 0; entry < PCB->LayerGroups.Number[group]; entry++)
 			if (PCB->LayerGroups.Entries[group][entry] == Layer)
 				return(group);
@@ -824,6 +858,10 @@ BoxTypePtr GetObjectBoundingBox(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 			break;
 		}
 
+		case ARC_TYPE:
+			box = ((ArcTypePtr) Ptr2)->BoundingBox; 
+			break;
+
 		case TEXT_TYPE:
 		case ELEMENTNAME_TYPE:
 			box = ((TextTypePtr) Ptr2)->BoundingBox;
@@ -878,11 +916,36 @@ BoxTypePtr GetObjectBoundingBox(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 		}
 
 		default:
-			return(NULL);
+			Message("Request for bounding box of unsupported typ %d\n",Type);
+			box.X1 = box.X2 = box.Y1 = box.Y2 = 0;
+			break;
 	}
 	return(&box);
 }
 
+/* ---------------------------------------------------------------------------
+ * computes the bounding box of an arc
+ */
+void SetArcBoundingBox(ArcTypePtr Arc)
+{
+	BoxTypePtr		box;
+	register Position	temp, width;
+
+	box = GetArcEnds(Arc);
+	temp = box->X1;
+	box->X1 = MIN(box->X1, box->X2);
+	box->X2 = MAX(box->X2, temp);
+	temp = box->Y1;
+	box->Y1 = MIN(box->Y1, box->Y2);
+	box->Y2 = MAX(box->Y2, temp);
+	width = Arc->Thickness/2;
+	box->X1 -= width;
+	box->X2 += width;
+	box->Y1 -= width;
+	box->Y2 += width;
+	Arc->BoundingBox = *box;
+}
+
 /* ---------------------------------------------------------------------------
  * resets the layerstack setting
  */
@@ -940,7 +1003,7 @@ char *GetWorkingDirectory(void)
 {
 	static	char	path[MAXPATHLEN+1];
 
-#ifdef SYSV
+#if defined(SYSV) || defined(linux) 
 	return(getcwd(path, MAXPATHLEN));
 #else
 		/* seems that some BSD releases lack of a prototype for getwd() */
@@ -965,3 +1028,122 @@ void CreateQuotedString(DynamicStringTypePtr DS, char *S)
 	DSAddCharacter(DS, '"');
 }
 
+/* ---------------------------------------------------------------------------
+ * returns the current possible grid factor or zero
+ */
+int GetGridFactor(void)
+{
+	static	int	factor[] = { 1, 2, 5, 10 };
+		int	i,
+			delta;
+
+		/* check to see if we have to draw every point,
+		 * every 2nd, 5th or 10th
+		 */
+	for (i = 0; i < ENTRIES(factor); i++)
+	{
+		delta = PCB->Grid *factor[i];
+		if (TO_SCREEN(delta) >= MIN_GRID_DISTANCE)
+		{
+			if (Settings.GridFactor != factor[i])
+			{
+				Settings.GridFactor = factor[i];
+				SetStatusLine();
+			}
+			return(factor[i]);
+		}
+	}
+	Settings.GridFactor = 0;
+	SetStatusLine();
+	return(0);
+}
+
+static	void RightAngles(int Angle, float *cosa, float *sina)
+{
+	switch (Angle)
+	{
+		case -360:
+			*cosa = 1.0;
+			*sina = 0;
+			break;
+		case -315:
+			*cosa = SQRT2OVER2;
+			*sina = SQRT2OVER2;
+			break;
+		case -270:
+			*cosa = 0;
+			*sina = 1.0;
+			break;
+		case -225:
+			*cosa = -SQRT2OVER2;
+			*sina = SQRT2OVER2;
+			break;
+		case -180:
+			*cosa = -1.0;
+			*sina = 0;
+			break;
+		case -135:
+			*cosa = -SQRT2OVER2;
+			*sina = -SQRT2OVER2;
+			break;
+		case -90:
+			*cosa = 0;
+			*sina = -1.0;
+			break;
+		case -45:
+			*cosa = SQRT2OVER2;
+			*sina = -SQRT2OVER2;
+			break;
+		case 0:
+			*cosa = 1.0;
+			*sina = 0;
+			break;
+		case 45:
+			*cosa = SQRT2OVER2;
+			*sina = SQRT2OVER2;
+			break;
+		case 90:
+			*cosa = 0;
+			*sina = 1.0;
+			break;
+		case 135:
+			*cosa = -SQRT2OVER2;
+			*sina = SQRT2OVER2;
+			break;
+		case 180:
+			*cosa = -1.0;
+			*sina = 0;
+			break;
+		case 225:
+			*cosa = -SQRT2OVER2;
+			*sina = -SQRT2OVER2;
+			break;
+		case 270:
+			*cosa = 0;
+			*sina = -1.0;
+			break;
+		case 315:
+			*cosa = SQRT2OVER2;
+			*sina = -SQRT2OVER2;
+			break;
+		case 360:
+			*cosa = 1.0;
+			*sina = 0;
+			break;
+	}
+}
+
+BoxTypePtr GetArcEnds(ArcTypePtr Arc)
+{
+	static	BoxType	box;
+	float	ca, sa;
+
+	RightAngles(Arc->StartAngle, &ca, &sa);
+	box.X1 = Arc->X - Arc->Width * ca;
+	box.Y1 = Arc->Y + Arc->Width * sa;
+	RightAngles(Arc->StartAngle + Arc->Delta, &ca, &sa);
+	box.X2 = Arc->X - Arc->Width * ca;
+	box.Y2 = Arc->Y + Arc->Width * sa;
+	return(&box);
+}
+
