@@ -22,7 +22,7 @@
  *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
  *  Thomas.Nau@rz.uni-ulm.de
  *
- *  RCS: $Id: misc.h,v 143.1 1996/09/16 09:08:46 nau Exp $
+ *  RCS: $Id: misc.h,v 1.2 1998/02/21 10:27:14 cad Exp $
  */
 
 /* prototypes for misc routines
@@ -35,31 +35,34 @@
 #include "global.h"
 #include "mymem.h"
 
-void			Copyright(void);
-void			Usage(void);
-void			SetPolygonBoundingBox(PolygonTypePtr);
-void			SetElementBoundingBox(ElementTypePtr);
-Boolean			IsDataEmpty(DataTypePtr);
-BoxTypePtr		GetDataBoundingBox(DataTypePtr);
-void			CenterDisplay(Position, Position);
-void			SetFontInfo(FontTypePtr);
-void			UpdateSettingsOnScreen(void);
-int				ParseGroupString(char *, LayerGroupTypePtr);
-void			QuitApplication(void);
-char			*EvaluateFilename(char *, char *, char *, char *);
-char			*ExpandFilename(char *, char *);
-void			SetTextBoundingBox(FontTypePtr, TextTypePtr);
-Boolean			GetPosition(char *);
-void			ReleaseSaveUnderPixmap(void);
-void			SaveOutputWindow(void);
+void		Copyright(void);
+void		Usage(void);
+void		SetPolygonBoundingBox(PolygonTypePtr);
+void		SetElementBoundingBox(ElementTypePtr, FontTypePtr);
+Boolean		IsDataEmpty(DataTypePtr);
+BoxTypePtr	GetDataBoundingBox(DataTypePtr);
+void		CenterDisplay(Position, Position);
+void		SetFontInfo(FontTypePtr);
+void		UpdateSettingsOnScreen(void);
+int		ParseGroupString(char *, LayerGroupTypePtr);
+void		QuitApplication(void);
+char		*EvaluateFilename(char *, char *, char *, char *);
+char		*ExpandFilename(char *, char *);
+void		SetTextBoundingBox(FontTypePtr, TextTypePtr);
+Boolean		GetPosition(char *);
+void		ReleaseSaveUnderPixmap(void);
+void		SaveOutputWindow(void);
 unsigned int	SetOutputXCursor(unsigned int);
-int				GetLayerNumber(DataTypePtr, LayerTypePtr);
-int				GetLayerGroupNumberByPointer(LayerTypePtr);
-int				GetLayerGroupNumberByNumber(Cardinal);
-BoxTypePtr		GetObjectBoundingBox(int, void *, void *, void *);
-void			ResetStackAndVisibility(void);
-void			ChangePCBSize(Dimension, Dimension);
-char			*GetWorkingDirectory(void);
-void			CreateQuotedString(DynamicStringTypePtr, char *);
+int		GetLayerNumber(DataTypePtr, LayerTypePtr);
+int		GetLayerGroupNumberByPointer(LayerTypePtr);
+int		GetLayerGroupNumberByNumber(Cardinal);
+BoxTypePtr	GetObjectBoundingBox(int, void *, void *, void *);
+void		ResetStackAndVisibility(void);
+void		ChangePCBSize(Dimension, Dimension);
+char		*GetWorkingDirectory(void);
+void		CreateQuotedString(DynamicStringTypePtr, char *);
+int		GetGridFactor(void);
+void		SetArcBoundingBox(ArcTypePtr);
+BoxTypePtr	GetArcEnds(ArcTypePtr);
 
 #endif
