@@ -22,7 +22,7 @@
  *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
  *  Thomas.Nau@rz.uni-ulm.de
  *
- *  RCS: $Id: polygon.h,v 143.1 1996/09/16 09:08:51 nau Exp $
+ *  RCS: $Id: polygon.h,v 1.1 1998/02/19 18:06:34 sync Exp $
  */
 
 /* prototypes for polygon editing routines
@@ -33,10 +33,13 @@
 
 #include "global.h"
 
-PointTypePtr	GetLowestDistancePolygonPoint(PolygonTypePtr,
+Cardinal	GetLowestDistancePolygonPoint(PolygonTypePtr,
 					Position, Position);
 void			GoToPreviousPoint(void);
 void			ClosePolygon(void);
 void			CopyAttachedPolygonToLayer(void);
+void			UpdatePIPFlags(PinTypePtr, ElementTypePtr,
+				       LayerTypePtr, Boolean);
+
 
 #endif
