@@ -22,7 +22,7 @@
  *  Thomas Nau, Schlehenweg 15, 88471 Baustetten, Germany
  *  Thomas.Nau@rz.uni-ulm.de
  *
- *  RCS: $Id: print.h,v 143.1 1996/09/16 09:08:52 nau Exp $
+ *  RCS: $Id: print.h,v 1.1 1998/02/19 18:06:34 sync Exp $
  */
 
 /* prototypes for printing routines
@@ -35,8 +35,8 @@
 
 #include "global.h"
 
-int		Print(char *, float, Boolean, Boolean, Boolean,
-			Boolean, Boolean, Boolean, Boolean, Boolean,
-			PrintDeviceTypePtr, MediaTypePtr, Position, Position);
+int	Print(char *, float, Boolean, Boolean, Boolean,
+		Boolean, Boolean, Boolean, Boolean, Boolean,
+		PrintDeviceTypePtr, MediaTypePtr, Position, Position, Boolean);
 
 #endif
