@@ -24,11 +24,15 @@
  *
  */
 
-static	char	*rcsid = "$Id: search.c,v 143.1 1996/09/16 09:08:57 nau Exp $";
+static	char	*rcsid = "$Id: search.c,v 1.6.1.1 1998/03/01 22:58:33 cad Exp $";
 
 /* search routines
  * some of the functions use dummy parameters
  */
+
+#define SearchRange 2	/* The fraction of an items width that the search point */
+			/* must be within for the search to succeed */
+
 #include <math.h>
 
 #include "global.h"
@@ -36,19 +40,26 @@ static	char	*rcsid = "$Id: search.c,v 143.1 1996/09/16 09:08:57 nau Exp $";
 #include "data.h"
 #include "draw.h"
 #include "find.h"
+#include "misc.h"
 #include "search.h"
 
 /* ---------------------------------------------------------------------------
  * some local identifiers
  */
 static	Position		PosX,		/* search position for subroutines */
-						PosY;
+				PosY;
+static	Dimension		SearchRadius;
+static	LayerTypePtr	SearchLayer;
 
 /* ---------------------------------------------------------------------------
  * some local prototypes
  */
 static	Boolean	SearchLineByPosition(LayerTypePtr *, LineTypePtr *,
 					LineTypePtr *);
+static	Boolean	SearchArcByPosition(LayerTypePtr *, ArcTypePtr *,
+					ArcTypePtr *);
+static	Boolean	SearchRatLineByPosition(LayerTypePtr *, LineTypePtr *,
+					LineTypePtr *);
 static	Boolean	SearchTextByPosition(LayerTypePtr *, TextTypePtr *,
 					TextTypePtr *);
 static	Boolean	SearchPolygonByPosition(LayerTypePtr *, PolygonTypePtr *,
@@ -56,15 +67,16 @@ static	Boolean	SearchPolygonByPosition(LayerTypePtr *, PolygonTypePtr *,
 static	Boolean	SearchPinByPosition(ElementTypePtr *, PinTypePtr *,
 					PinTypePtr *);
 static	Boolean	SearchPadByPosition(ElementTypePtr *, PadTypePtr *,
-					PadTypePtr *);
+					PadTypePtr *, Boolean);
 static	Boolean	SearchViaByPosition(PinTypePtr *, PinTypePtr *, PinTypePtr *);
 static	Boolean	SearchElementNameByPosition(ElementTypePtr *, TextTypePtr *,
-					TextTypePtr *);
+					TextTypePtr *, Boolean);
 static	Boolean	SearchLinePointByPosition(LayerTypePtr *, LineTypePtr *,
 					PointTypePtr *);
 static	Boolean	SearchPointByPosition(LayerTypePtr *, PolygonTypePtr *,
 					PointTypePtr *);
-
+static Boolean SearchElementByPosition(ElementTypePtr *, ElementTypePtr *,
+					 ElementTypePtr *, Boolean);
 
 /* ---------------------------------------------------------------------------
  * searches a via
@@ -75,8 +87,8 @@ static Boolean SearchViaByPosition(PinTypePtr *Via, PinTypePtr *Dummy1,
 		/* search only if via-layer is visible */
 	if (PCB->ViaOn)
 		VIA_LOOP(PCB->Data,
-			if (abs(via->X - PosX) <= via->Thickness/3 &&
-		    	abs(via->Y - PosY) <= via->Thickness/3 )
+			if (abs(via->X - PosX) <= via->Thickness/SearchRange + SearchRadius &&
+		    	abs(via->Y - PosY) <= via->Thickness/SearchRange + SearchRadius)
 			{
 				*Via = *Dummy1 = *Dummy2 = via;
 				return(True);
@@ -96,8 +108,8 @@ static Boolean SearchPinByPosition(ElementTypePtr *Element, PinTypePtr *Pin,
 	if (PCB->PinOn)
 		ELEMENT_LOOP(PCB->Data,
 			PIN_LOOP(element,
-				if (abs(pin->X - PosX) <= pin->Thickness/3 &&
-					abs(pin->Y - PosY) <= pin->Thickness/3 )
+				if (abs(pin->X - PosX) <= pin->Thickness/SearchRange  + SearchRadius &&
+					abs(pin->Y - PosY) <= pin->Thickness/SearchRange + SearchRadius )
 				{
 						/* bug-fix by Ulrich Pegelow (ulrpeg@bigcomm.gun.de) */
 					*Element = element;
@@ -114,21 +126,34 @@ static Boolean SearchPinByPosition(ElementTypePtr *Element, PinTypePtr *Pin,
  * starts with the newest element
  */
 static Boolean SearchPadByPosition(ElementTypePtr *Element, PadTypePtr *Pad,
-	PadTypePtr *Dummy)
+	PadTypePtr *Dummy, Boolean BackToo)
 {
 		/* search only if pin-layer is visible */
 	if (PCB->PinOn)
 		ALLPAD_LOOP(PCB->Data,
 			if ((TEST_FLAG(ONSOLDERFLAG, pad) != 0) == SWAP_IDENT ||
-				PCB->InvisibleObjectsOn)
+				(BackToo & PCB->InvisibleObjectsOn))
 			{
 					/* the cast isn't very nice but working, check
 					 * global.h for details
 					 */
-				if (IsPointOnLine(PosX, PosY, 0, (LineTypePtr) pad))
+				if (TEST_FLAG(SQUAREFLAG, pad))
 				{
-					*Pad = *Dummy = pad;
-					return(True);
+					if (IsPointInSquarePad(PosX, PosY, SearchRadius, pad))
+					{
+						*Element = element;
+						*Pad = *Dummy = pad;
+						return(True);
+					}
+				}
+				else
+				{
+					if (IsPointOnLine(PosX, PosY, SearchRadius, (LineTypePtr) pad))
+					{
+						*Element = element;
+						*Pad = *Dummy = pad;
+						return(True);
+					}
 				}
 			}
 		);
@@ -136,113 +161,129 @@ static Boolean SearchPadByPosition(ElementTypePtr *Element, PadTypePtr *Pad,
 }
 
 /* ---------------------------------------------------------------------------
- * searches line on all layers that are switched on in layerstack order
+ * searches ordinary line on the SearchLayer 
  */
 static Boolean SearchLineByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 	LineTypePtr *Dummy)
 {
-	Cardinal		i;
-
-	for (i = 0; i < MAX_LAYER; i++)
-	{
-		*Layer = LAYER_ON_STACK(i);
-		if ((*Layer)->On)
-			LINE_LOOP(*Layer,
-				if (IsPointOnLine(PosX, PosY, 0, line))
-				{
-					*Line = *Dummy = line;
-					return(True);
-				}
-			);
-	}
+	*Layer = SearchLayer;
+	LINE_LOOP(*Layer,
+		if (!TEST_FLAG(RATFLAG, line) && IsPointOnLine(PosX, PosY, SearchRadius, line))
+		{
+			*Line = *Dummy = line;
+			return(True);
+		}
+	);
 	return(False);
 }
 
 /* ---------------------------------------------------------------------------
- * searches text on all layers that are switched on in layerstack order
+ * searches rat line on the SearchLayer 
+ */
+static Boolean SearchRatLineByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
+	LineTypePtr *Dummy)
+{
+	*Layer = SearchLayer;
+	LINE_LOOP(*Layer,
+		if (TEST_FLAG(RATFLAG, line) && IsPointOnLine(PosX, PosY, SearchRadius, line))
+		{
+			*Line = *Dummy = line;
+			return(True);
+		}
+	);
+	return(False);
+}
+
+/* ---------------------------------------------------------------------------
+ * searches arc on the SearchLayer 
+ */
+static Boolean SearchArcByPosition(LayerTypePtr *Layer, ArcTypePtr *Arc,
+	ArcTypePtr *Dummy)
+{
+	*Layer = SearchLayer;
+	ARC_LOOP(*Layer,
+		if(IsPointOnArc(PosX, PosY, SearchRadius, arc))
+		{
+			*Arc = *Dummy = arc;
+			return(True);
+		}
+	);
+	return(False);
+}
+
+/* ---------------------------------------------------------------------------
+ * searches text on the SearchLayer
  */
 static Boolean SearchTextByPosition(LayerTypePtr *Layer, TextTypePtr *Text,
 	TextTypePtr *Dummy)
 {
-	Cardinal		i;
-
-	for (i = 0; i < MAX_LAYER; i++)
-	{
-		*Layer = LAYER_ON_STACK(i);
-		if ((*Layer)->On)
-			TEXT_LOOP(*Layer,
-				if (POINT_IN_BOX(PosX, PosY, &text->BoundingBox))
-				{
-					*Text = *Dummy = text;
-					return(True);
-				}
-			);
-	}
+	*Layer = SearchLayer;
+	TEXT_LOOP(*Layer,
+		if (TEXT_IS_VISIBLE(PCB, *Layer, text) &&
+		    POINT_IN_BOX(PosX, PosY, &text->BoundingBox))
+		{
+			*Text = *Dummy = text;
+			return(True);
+		}
+	);
 	return(False);
 }
 
 /* ---------------------------------------------------------------------------
- * searches a polygon on all layers that are switched on in layerstack order
+ * searches a polygon on the SearchLayer 
  */
 static Boolean SearchPolygonByPosition(LayerTypePtr *Layer,
 	PolygonTypePtr *Polygon, PolygonTypePtr *Dummy)
 {
-	Cardinal		i;
-
-	for (i = 0; i < MAX_LAYER; i++)
-	{
-		*Layer = LAYER_ON_STACK(i);
-		if ((*Layer)->On)
-			POLYGON_LOOP(*Layer,
-				if (IsPointInPolygon(PosX, PosY, 0, polygon))
-				{
-					*Polygon = *Dummy = polygon;
-					return(True);
-				}
-			);
-	}
+	*Layer = SearchLayer;
+	POLYGON_LOOP(*Layer,
+		if (IsPointInPolygon(PosX, PosY, SearchRadius, polygon))
+		{
+			*Polygon = *Dummy = polygon;
+			return(True);
+		}
+	);
 	return(False);
 }
 
 /* ---------------------------------------------------------------------------
- * searches a line-point on all layers that are switched on
- * in layerstack order
- *
+ * searches a line-point on all the search layer
  */
 static Boolean SearchLinePointByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 	PointTypePtr *Point)
 {
-	Cardinal		i;
+	float			d, least;
+	Boolean			found = False;
 
-	for (i = 0; i < MAX_LAYER; i++)
-	{
-		float			d;
-
-		*Layer = LAYER_ON_STACK(i);
-		if ((*Layer)->On)
-			LINE_LOOP(*Layer,
+	least = (MAX_LINE_POINT_DISTANCE + SearchRadius) * (MAX_LINE_POINT_DISTANCE + SearchRadius);
+	*Layer = SearchLayer;
+	LINE_LOOP(*Layer,
+		if (!TEST_FLAG(RATFLAG, line))
+		{
+				/* some stupid code to check both points */
+			d = (PosX - line->Point1.X)*(PosX - line->Point1.X)+
+				(PosY - line->Point1.Y)*(PosY - line->Point1.Y);
+			if (d < least)
 			{
-					/* some stupid code to check both points */
-				d = (PosX - line->Point1.X)*(PosX - line->Point1.X)+
-					(PosY - line->Point1.Y)*(PosY - line->Point1.Y);
-				if (d < MAX_LINE_POINT_DISTANCE*MAX_LINE_POINT_DISTANCE)
-				{
-					*Line = line;
-					*Point = &line->Point1;
-					return(True);
-				}
-
-				d = (PosX - line->Point2.X)*(PosX - line->Point2.X)+
-					(PosY - line->Point2.Y)*(PosY - line->Point2.Y);
-				if (d < MAX_LINE_POINT_DISTANCE*MAX_LINE_POINT_DISTANCE)
-				{
-					*Line = line;
-					*Point = &line->Point2;
-					return(True);
-				}
+				least = d;
+				*Line = line;
+				*Point = &line->Point1;
+				found = True;
 			}
-			);
-	}
+
+			d = (PosX - line->Point2.X)*(PosX - line->Point2.X)+
+				(PosY - line->Point2.Y)*(PosY - line->Point2.Y);
+			if (d < least)
+			{
+				least = d;
+				*Line = line;
+				*Point = &line->Point2;
+				found = True;
+			}
+		}
+	    );
+		/* return with nearest */
+	if (found) return(True);
 	return(False);
 }
 
@@ -253,29 +294,27 @@ static Boolean SearchLinePointByPosition(LayerTypePtr *Layer, LineTypePtr *Line,
 static Boolean SearchPointByPosition(LayerTypePtr *Layer,
 	PolygonTypePtr *Polygon, PointTypePtr *Point)
 {
-	Cardinal	i;
+	float			d, least;
+	Boolean			found = False;
 
-	for (i = 0; i < MAX_LAYER; i++)
-	{
-		float			d;
-
-		*Layer = LAYER_ON_STACK(i);
-		if ((*Layer)->On)
-			POLYGON_LOOP(*Layer,
-				POLYGONPOINT_LOOP(polygon,
-				{
-					d = (point->X - PosX)*(point->X - PosX)+
-						(point->Y - PosY)*(point->Y - PosY);
-					if (d < MAX_POLYGON_POINT_DISTANCE*MAX_POLYGON_POINT_DISTANCE)
-					{
-						*Polygon = polygon;
-						*Point = point;
-						return(True);
-					}
-				}
-				);
-			);
-	}
+	least = (SearchRadius + MAX_POLYGON_POINT_DISTANCE) * (SearchRadius + MAX_POLYGON_POINT_DISTANCE);
+	*Layer = SearchLayer;
+	POLYGON_LOOP(*Layer,
+		POLYGONPOINT_LOOP(polygon,
+		{
+			d = (point->X - PosX)*(point->X - PosX)+
+				(point->Y - PosY)*(point->Y - PosY);
+			if (d < least)
+			{
+				least = d;
+				*Polygon = polygon;
+				*Point = point;
+				found = True;
+			}
+		}
+		);
+	);
+	if (found) return(True);
 	return(False);
 }
 
@@ -284,7 +323,7 @@ static Boolean SearchPointByPosition(LayerTypePtr *Layer,
  * the search starts with the last element and goes back to the beginning
  */
 static Boolean SearchElementNameByPosition(ElementTypePtr *Element,
-	TextTypePtr *Text, TextTypePtr *Dummy)
+	TextTypePtr *Text, TextTypePtr *Dummy, Boolean BackToo)
 {
 	TextTypePtr		text;
 
@@ -292,8 +331,8 @@ static Boolean SearchElementNameByPosition(ElementTypePtr *Element,
 	if (PCB->ElementOn)
 	{
 		ELEMENT_LOOP(PCB->Data,
-			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-				PCB->InvisibleObjectsOn)
+			if (((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
+				(BackToo && PCB->InvisibleObjectsOn)) && !TEST_FLAG(HIDENAMEFLAG, element))
 			{
 				text = &ELEMENT_TEXT(PCB, element);
 				if (POINT_IN_BOX(PosX, PosY, &text->BoundingBox))
@@ -314,14 +353,14 @@ static Boolean SearchElementNameByPosition(ElementTypePtr *Element,
  * if more than one element matches, the smallest one is taken
  */
 static Boolean SearchElementByPosition(ElementTypePtr *Element,
-	ElementTypePtr *Dummy1, ElementTypePtr *Dummy2)
+	ElementTypePtr *Dummy1, ElementTypePtr *Dummy2, Boolean BackToo)
 {
 	ElementTypePtr	save = NULL;
 	long	 		area = 0;
 	Boolean			found;
 
-		/* package layer have to be switched on */
-	if (PCB->ElementOn || PCB->PinOn)
+		/* Both package layers have to be switched on */
+	if (PCB->ElementOn && PCB->PinOn)
 	{
 			/* the element names bounding box is not necessarily
 			 * a part of the elements bounding box;
@@ -329,12 +368,12 @@ static Boolean SearchElementByPosition(ElementTypePtr *Element,
 			 */
 		ELEMENT_LOOP(PCB->Data,
 			if ((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-				PCB->InvisibleObjectsOn)
+				(BackToo && PCB->InvisibleObjectsOn))
 			{
 				found = POINT_IN_BOX(PosX, PosY, &element->BoundingBox);
-				ELEMENTTEXT_LOOP(element,
-					found |= POINT_IN_BOX(PosX, PosY, &text->BoundingBox);
-				);
+				if (!TEST_FLAG(HIDENAMEFLAG, element))
+						found |= POINT_IN_BOX(PosX, PosY,
+						         &element->Name[NAME_INDEX(PCB)].BoundingBox);
 				if (found)
 				{
 					long	newarea;
@@ -355,6 +394,17 @@ static Boolean SearchElementByPosition(ElementTypePtr *Element,
 	return(save != NULL);
 }
 
+/* ---------------------------------------------------------------------------
+ * checks if a line end is on a PV
+ */
+Boolean IsPointOnLineEnd(Position X, Position Y, LineTypePtr Line)
+{
+	if (((X == Line->Point1.X) && (Y == Line->Point1.Y)) ||
+	   ((X == Line->Point2.X) && (Y == Line->Point2.Y)))
+		return(True);
+	return(False);
+}
+
 /* ---------------------------------------------------------------------------
  * checks if a line intersects with a PV
  * constant recognition by the optimizer is assumed
@@ -436,6 +486,7 @@ Boolean IsLineInRectangle(Position X1, Position Y1,
 
 		/* construct a set of dummy lines and check each of them */
 	line.Thickness = 0;
+	line.Flags = NOFLAG;
 
 		/* upper-left to upper-right corner */
 	line.Point1.Y = line.Point2.Y = Y1;
@@ -480,6 +531,64 @@ Boolean IsLineInRectangle(Position X1, Position Y1,
 	return(False);
 }
 
+/* ---------------------------------------------------------------------------
+ * checks if an arc crosses a square
+ */
+Boolean IsArcInRectangle(Position X1, Position Y1,
+	Position X2, Position Y2, ArcTypePtr Arc)
+{
+	LineType	line;
+	Dimension	thick = Arc->Thickness/2 +1;
+
+		/* construct a set of dummy lines and check each of them */
+	line.Thickness = thick;
+	line.Flags = NOFLAG;
+
+		/* upper-left to upper-right corner */
+	line.Point1.Y = line.Point2.Y = Y1;
+	line.Point1.X = X1;
+	line.Point2.X = X2;
+	if (LineArcIntersect(&line, Arc))
+			return(True);
+
+		/* upper-right to lower-right corner */
+	line.Point1.X = line.Point2.X = X2;
+	line.Point1.Y = Y1;
+	line.Point2.Y = Y2;
+	if (LineArcIntersect(&line, Arc))
+		return(True);
+
+		/* lower-right to lower-left corner */
+	line.Point1.Y = line.Point2.Y = Y2;
+	line.Point1.X = X1;
+	line.Point2.X = X2;
+	if (LineArcIntersect(&line, Arc))
+		return(True);
+
+		/* lower-left to upper-left corner */
+	line.Point1.X = line.Point2.X = X1;
+	line.Point1.Y = Y1;
+	line.Point2.Y = Y2;
+	if (LineArcIntersect(&line, Arc))
+		return(True);
+
+	return(False);
+}
+
+/* ---------------------------------------------------------------------------
+ * check if a point lies inside a square Pad
+ */
+Boolean IsPointInSquarePad(Position X, Position Y, Cardinal Radius, PadTypePtr Pad)
+{
+	BoxType		padbox;
+
+	padbox.X1 = MIN(Pad->Point1.X, Pad->Point2.X) - Pad->Thickness/2 - Radius;
+	padbox.X2 = MAX(Pad->Point1.X, Pad->Point2.X) + Pad->Thickness/2 + Radius;
+	padbox.Y1 = MIN(Pad->Point1.Y, Pad->Point2.Y) - Pad->Thickness/2 - Radius;
+	padbox.Y2 = MAX(Pad->Point1.Y, Pad->Point2.Y) + Pad->Thickness/2 + Radius;
+	return(POINT_IN_BOX(X, Y, &padbox));
+}
+
 /* ---------------------------------------------------------------------------
  * checks if a point lies inside a polygon
  * the code assumes that the last point isn't equal to the first one
@@ -529,6 +638,7 @@ Boolean IsPointInPolygon(float X, float Y, float Radius, PolygonTypePtr Polygon)
 
 			line.Point1 = Polygon->Points[0];
 			line.Thickness = 0;
+			line.Flags = NOFLAG;
 
 				/* POLYGONPOINT_LOOP decrements pointers !!! */
 			POLYGONPOINT_LOOP(Polygon,
@@ -544,7 +654,7 @@ Boolean IsPointInPolygon(float X, float Y, float Radius, PolygonTypePtr Polygon)
 }
 
 /* ---------------------------------------------------------------------------
- * checks is a polygon intersects with a square
+ * checks if a polygon intersects with a square
  */
 Boolean IsRectangleInPolygon(Position X1, Position Y1,
 	Position X2, Position Y2, PolygonTypePtr Polygon)
@@ -562,6 +672,30 @@ Boolean IsRectangleInPolygon(Position X1, Position Y1,
 	return(IsPolygonInPolygon(&polygon, Polygon));
 }
 
+Boolean IsPointOnArc(float X, float Y, float Radius, ArcTypePtr Arc)
+{
+	register float r, wt, sr, x1, y1, x2, y2;
+	BoxTypePtr box;
+
+	r = (X - Arc->X)*(X - Arc->X) + (Y - Arc->Y)*(Y - Arc->Y);
+		/* sr is the search radius */
+	sr = 0.5 * Arc->Thickness + Radius;
+	wt = Arc->Width * sr * 2.0;
+	if (abs(r - (Arc->Width * Arc->Width + sr * sr)) > wt)
+		return(False);
+	box = GetArcEnds(Arc);
+	if (X >= MIN(box->X1, box->X2) && X <= MAX(box->X1, box->X2)
+		&& Y >= MIN(box->Y1, box->Y2) && Y <= MAX(box->Y1, box->Y2))
+		return(True);
+		/* check the end points for the search radius */
+	sr *= sr;
+	if ((X - box->X1)*(X - box->X1) + (Y - box->Y1)*(Y - box->Y1) <= sr)
+		return(True);
+	if ((X - box->X2)*(X - box->X2) + (Y - box->Y2)*(Y - box->Y2) <= sr)
+		return(True);
+	return(False);
+}
+
 /* ---------------------------------------------------------------------------
  * searches for any kind of object or for a set of object types
  * the calling routine passes two pointers to allocated memory for storing
@@ -575,62 +709,127 @@ Boolean IsRectangleInPolygon(Position X1, Position Y1,
  */
 int SearchObjectByPosition(int Type,
 	void **Result1, void **Result2, void **Result3,
-	Position X, Position Y)
+	Position X, Position Y, Dimension Radius)
 {
+	Cardinal i;
+	int HigherBound = 0;
+	Boolean HigherAvail = False;
 		/* setup local identifiers */
 	PosX = X;
 	PosY = Y;
+	SearchRadius = Radius;
 
-	if (Type & LINEPOINT_TYPE &&
-		SearchLinePointByPosition((LayerTypePtr *) Result1,
-			(LineTypePtr *) Result2,
-			(PointTypePtr *) Result3))
-		return(LINEPOINT_TYPE);
-	
-	if (Type & POLYGONPOINT_TYPE &&
-		SearchPointByPosition((LayerTypePtr *) Result1,
-			(PolygonTypePtr *) Result2,
-			(PointTypePtr *) Result3))
-		return(POLYGONPOINT_TYPE);
+	if (Type & VIA_TYPE &&
+		SearchViaByPosition((PinTypePtr *) Result1,
+			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
+		return(VIA_TYPE);
 
 	if (Type & PIN_TYPE &&
+		SearchPinByPosition((ElementTypePtr *) Result1,
+			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
+		HigherAvail = True;
+	
+	if (!HigherAvail && Type & PAD_TYPE &&
+		SearchPadByPosition((ElementTypePtr *) Result1,
+			(PadTypePtr *) Result2, (PadTypePtr *) Result3, False))
+		HigherAvail = True;
+	
+	if (!HigherAvail && Type & ELEMENTNAME_TYPE &&
+		SearchElementNameByPosition((ElementTypePtr *) Result1,
+			(TextTypePtr *) Result2, (TextTypePtr *) Result3, False))
+	{
+		BoxTypePtr box = &(*(TextTypePtr *) Result2)->BoundingBox;
+		HigherBound = (box->X2 - box->X1) * (box->Y2 - box->Y1);
+		HigherAvail = True;
+	}
+
+	if (!HigherAvail && Type & ELEMENT_TYPE &&
+		SearchElementByPosition((ElementTypePtr *) Result1,
+			(ElementTypePtr *) Result2, (ElementTypePtr *) Result3, False))
+	{
+		BoxTypePtr box = &(*(ElementTypePtr *) Result1)->BoundingBox;
+		HigherBound = (box->X2 - box->X1) * (box->Y2 - box->Y1);
+		HigherAvail = True;
+	}
+
+	for (i = 0; i < MAX_LAYER; i++)
+	{
+		SearchLayer = LAYER_ON_STACK(i);
+		if (SearchLayer->On)
+		{
+			if (Type & POLYGONPOINT_TYPE &&
+				SearchPointByPosition((LayerTypePtr *) Result1,
+					(PolygonTypePtr *) Result2, (PointTypePtr *) Result3))
+				return(POLYGONPOINT_TYPE);
+
+			if (Type & POLYGON_TYPE &&
+				SearchPolygonByPosition((LayerTypePtr *) Result1,
+					(PolygonTypePtr *) Result2, (PolygonTypePtr *) Result3))
+				if (HigherAvail)
+				{
+					BoxTypePtr box = &(*(PolygonTypePtr *) Result2)->BoundingBox;
+					int area = (box->X2 - box->X1) * (box->X2 - box->X1);
+					if (HigherBound < area)
+						break;
+					else
+						return(POLYGON_TYPE);
+				}
+				else
+					return(POLYGON_TYPE);
+
+			if (Type & LINEPOINT_TYPE &&
+				SearchLinePointByPosition((LayerTypePtr *) Result1,
+					(LineTypePtr *) Result2,
+					(PointTypePtr *) Result3))
+				return(LINEPOINT_TYPE);
+	
+			if (Type & LINE_TYPE &&
+				SearchLineByPosition((LayerTypePtr *) Result1,
+					(LineTypePtr *) Result2, (LineTypePtr *) Result3))
+				return(LINE_TYPE);
+
+			if (Type & ARC_TYPE &&
+				SearchArcByPosition((LayerTypePtr *) Result1,
+					(ArcTypePtr *) Result2, (ArcTypePtr *) Result3))
+				return(ARC_TYPE);
+
+			if (Type & TEXT_TYPE &&
+				SearchTextByPosition((LayerTypePtr *) Result1,
+					(TextTypePtr *) Result2, (TextTypePtr *) Result3))
+				return(TEXT_TYPE);
+		}
+	}
+
+	if (HigherAvail && Type & PIN_TYPE &&
 		SearchPinByPosition((ElementTypePtr *) Result1,
 			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
 		return(PIN_TYPE);
 	
 	if (Type & PAD_TYPE &&
 		SearchPadByPosition((ElementTypePtr *) Result1,
-			(PadTypePtr *) Result2, (PadTypePtr *) Result3))
+			(PadTypePtr *) Result2, (PadTypePtr *) Result3, True))
 		return(PAD_TYPE);
 	
-	if (Type & VIA_TYPE &&
-		SearchViaByPosition((PinTypePtr *) Result1,
-			(PinTypePtr *) Result2, (PinTypePtr *) Result3))
-		return(VIA_TYPE);
-
-	if (Type & LINE_TYPE &&
-		SearchLineByPosition((LayerTypePtr *) Result1,
-			(LineTypePtr *) Result2, (LineTypePtr *) Result3))
-		return(LINE_TYPE);
-
-	if (Type & TEXT_TYPE &&
-		SearchTextByPosition((LayerTypePtr *) Result1,
-			(TextTypePtr *) Result2, (TextTypePtr *) Result3))
-		return(TEXT_TYPE);
-
 	if (Type & ELEMENTNAME_TYPE &&
 		SearchElementNameByPosition((ElementTypePtr *) Result1,
-			(TextTypePtr *) Result2, (TextTypePtr *) Result3))
+			(TextTypePtr *) Result2, (TextTypePtr *) Result3, True))
 		return(ELEMENTNAME_TYPE);
 
-	if (Type & POLYGON_TYPE &&
-		SearchPolygonByPosition((LayerTypePtr *) Result1,
-			(PolygonTypePtr *) Result2, (PolygonTypePtr *) Result3))
-		return(POLYGON_TYPE);
+	if (Type & RATLINE_TYPE)
+		for (i = 0; i < MAX_LAYER; i++)
+		{
+			SearchLayer = LAYER_ON_STACK(i);
+			if (SearchLayer->On)
+			{
+				if (SearchRatLineByPosition((LayerTypePtr *) Result1,
+						(LineTypePtr *) Result2, (LineTypePtr *) Result3))
+					return(LINE_TYPE);
+			}
+		}
 
 	if (Type & ELEMENT_TYPE &&
 		SearchElementByPosition((ElementTypePtr *) Result1,
-			(ElementTypePtr *) Result2, (ElementTypePtr *) Result3))
+			(ElementTypePtr *) Result2, (ElementTypePtr *) Result3, True))
 		return(ELEMENT_TYPE);
 
 	return(NO_TYPE);
@@ -669,6 +868,14 @@ int SearchObjectByID(DataTypePtr Base,
 			return(LINEPOINT_TYPE);
 		}
 	);
+	ALLARC_LOOP(Base,
+		if (arc->ID == ID)
+		{
+			*Result1 = (void *) layer;
+			*Result2 = *Result3 = (void *) arc;
+			return(ARC_TYPE);
+		}
+	);
 
 	ALLTEXT_LOOP(Base,
 		if (text->ID == ID)
@@ -707,15 +914,15 @@ int SearchObjectByID(DataTypePtr Base,
 
 		/* check pins and elementnames too */
 	ELEMENT_LOOP(Base,
-		*Result1 = *Result2 = *Result3 = (void *) element;
 		if (element->ID == ID)
 		{
-			*Result2 = *Result3 = (void *) element;
+			*Result1 = *Result2 = *Result3 = (void *) element;
 			return(ELEMENT_TYPE);
 		}
 		ELEMENTTEXT_LOOP(element,
 			if (text->ID == ID)
 			{
+				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) text;
 				return(ELEMENTNAME_TYPE);
 			}
@@ -723,6 +930,7 @@ int SearchObjectByID(DataTypePtr Base,
 		PIN_LOOP(element,
 			if (pin->ID == ID)
 			{
+				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) pin;
 				return(PIN_TYPE);
 			}
@@ -730,6 +938,7 @@ int SearchObjectByID(DataTypePtr Base,
 		PAD_LOOP(element,
 			if (pad->ID == ID)
 			{
+				*Result1 = (void *) element;
 				*Result2 = *Result3 = (void *) pad;
 				return(PAD_TYPE);
 			}
@@ -738,3 +947,23 @@ int SearchObjectByID(DataTypePtr Base,
 
 	return(NO_TYPE);
 }
+
+/* ---------------------------------------------------------------------------
+ * searches for an element by its board name.
+ * The function returns a pointer to the element, NULL if not found
+ */
+ElementTypePtr SearchElementByName(DataTypePtr Base, char *Name)
+{
+	ElementTypePtr	result = NULL;
+	
+	ELEMENT_LOOP(Base,
+		if (element->Name[1].TextString &&
+		    strcmp(element->Name[1].TextString, Name) == 0)
+		{
+			result = element;
+			return(result);
+		}
+	);
+	return result;
+}
+
