@@ -24,7 +24,7 @@
  *
  */
 
-static	char	*rcsid = "$Id: undo.c,v 143.1 1996/09/16 09:09:00 nau Exp $";
+static	char	*rcsid = "$Id: undo.c,v 1.6 1998/02/24 22:54:47 cad Exp $";
 
 /* functions used to undo operations
  *
@@ -66,6 +66,11 @@ static	char	*rcsid = "$Id: undo.c,v 143.1 1996/09/16 09:09:00 nau Exp $";
 /* ---------------------------------------------------------------------------
  * some local data types
  */
+typedef struct
+{
+	BoxType			*Box;		/* drawing box for block operations */
+} BlockType, *BlockTypePtr;
+
 typedef struct					/* information about a change command */
 {
 	char			*Name;
@@ -107,6 +112,9 @@ typedef	struct					/* holds information about an operation */
 		RemovedPointType	RemovedPoint;
 		RotateType			Rotate;
 		MoveToLayerType		MoveToLayer;
+		int			Flags;
+		Dimension		Size;
+		BlockType		Block;
 	} Data;
 } UndoListType, *UndoListTypePtr;
 
@@ -117,7 +125,7 @@ static	DataTypePtr		RemoveList;		/* list of removed objects */
 static	UndoListTypePtr	UndoList;		/* list of operations */
 static	int				Serial,			/* serial number */
 						SavedSerial;
-static	size_t			UndoN,			/* number of entries */
+static	size_t			UndoN, RedoN,			/* number of entries */
 						UndoMax;
 static	Boolean			Locked = False;	/* do not add entries if */
 										/* flag is set; prevents from */
@@ -136,13 +144,19 @@ static	Boolean			UndoRemove(UndoListTypePtr);
 static	Boolean			UndoRemovePoint(UndoListTypePtr);
 static	Boolean			UndoInsertPoint(UndoListTypePtr);
 static	Boolean			UndoMoveToLayer(UndoListTypePtr);
+static	Boolean			UndoFlag(UndoListTypePtr);
+static	Boolean			UndoChangeSize(UndoListTypePtr);
+static	Boolean			UndoChange2ndSize(UndoListTypePtr);
+static	Boolean			UndoBlock(UndoListTypePtr);
 
 /* ---------------------------------------------------------------------------
  * adds a command plus some data to the undo list
  */
 static UndoListTypePtr GetUndoSlot(int CommandType, int ID)
 {
-			UndoListTypePtr	ptr;
+	UndoListTypePtr	ptr;
+	void	*ptr1, *ptr2, *ptr3;
+	int	type;
 	static	size_t			limit = UNDO_WARNING_SIZE;
 
 		/* allocate memory */
@@ -165,6 +179,30 @@ static UndoListTypePtr GetUndoSlot(int CommandType, int ID)
 		}
 	}
 
+		/* free structures from the pruned redo list */
+
+	for (ptr = &UndoList[UndoN]; RedoN; ptr++, RedoN--)
+		switch(ptr->Type)
+		{
+			case UNDO_CHANGENAME:
+				SaveFree(ptr->Data.ChangeName.Name);
+				break;
+			case UNDO_BLOCK:
+				SaveFree(ptr->Data.Block.Box);
+				break;
+			case UNDO_REMOVE:
+				type = SearchObjectByID(RemoveList, &ptr1,
+					&ptr2, &ptr3, ptr->ID);
+				if (type != NO_TYPE)
+				{
+					DestroyObject(RemoveList, type,
+						ptr1, ptr2, ptr3);
+				}
+				break;
+			default:
+				break;
+		}		
+
 		/* copy typefield and serial number to the list */
 	ptr = &UndoList[UndoN++]; 
 	ptr->Type = CommandType;
@@ -178,69 +216,15 @@ static UndoListTypePtr GetUndoSlot(int CommandType, int ID)
  */
 static void DrawRecoveredObject(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 {
-	if (Type & (LINE_TYPE | TEXT_TYPE | POLYGON_TYPE))
+	if (Type & (LINE_TYPE | TEXT_TYPE | POLYGON_TYPE | ARC_TYPE))
 	{
 		LayerTypePtr	layer;
 
 		layer= &PCB->Data->Layer[GetLayerNumber(RemoveList,(LayerTypePtr)Ptr1)];
-		if (layer->On)
-			switch(Type)
-			{
-				case LINE_TYPE:
-					DrawLine(layer, (LineTypePtr) Ptr2);
-					break;
-
-				case TEXT_TYPE:
-					DrawText(layer, (TextTypePtr) Ptr2);
-					break;
-
-				case POLYGON_TYPE:
-					DrawPolygon(layer, (PolygonTypePtr) Ptr2);
-					break;
-			}
+		DrawObject(Type, (void *) layer, Ptr2, 0);
 	}
 	else
-		switch(Type)
-		{
-			case ELEMENT_TYPE:
-			{
-				ElementTypePtr	element = (ElementTypePtr) Ptr2;
-
-				if (PCB->ElementOn &&
-					((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-					PCB->InvisibleObjectsOn))
-					DrawElement(element);
-				break;
-			}
-
-			case ELEMENTNAME_TYPE:
-			{
-				ElementTypePtr	element = (ElementTypePtr) Ptr1;
-
-				if (PCB->ElementOn &&
-					((TEST_FLAG(ONSOLDERFLAG, element) != 0) == SWAP_IDENT ||
-					PCB->InvisibleObjectsOn))
-					DrawElementName(element);
-				break;
-			}
-
-			case PIN_TYPE:
-				if (PCB->PinOn)
-					DrawPin((PinTypePtr) Ptr2);
-				break;
-
-			case PAD_TYPE:
-				if (PCB->PinOn &&
-					((TEST_FLAG(ONSOLDERFLAG, (PadTypePtr) Ptr2) != 0) == SWAP_IDENT ||
-					PCB->InvisibleObjectsOn))
-					DrawPad((PadTypePtr) Ptr2);
-				break;
-
-			case VIA_TYPE:
-				if (PCB->ViaOn)
-					DrawVia((PinTypePtr) Ptr2);
-				break;
-		}
+		DrawObject(Type, Ptr1, Ptr2, 0);
 }
 
 /* ---------------------------------------------------------------------------
@@ -259,11 +243,25 @@ static Boolean UndoRotate(UndoListTypePtr Entry)
 		RotateObject(type, ptr1, ptr2, ptr3,
 			Entry->Data.Rotate.CenterX, Entry->Data.Rotate.CenterY,
 			(4-Entry->Data.Rotate.Steps) & 0x03);
+		Entry->Data.Rotate.Steps = (4 - Entry->Data.Rotate.Steps) & 0x03;
 		return(True);
 	}
 	return(False);
 }
 
+/* handles the block drawing aspect of undoing */
+static Boolean UndoBlock(UndoListTypePtr Entry)
+{
+
+	if (Entry->Data.Block.Box)
+	{
+		DrawBlock(Entry->Data.Block.Box);
+		BlockDrawing = False;
+	}
+	else
+		BlockDrawing = True;
+}
+
 /* ---------------------------------------------------------------------------
  * recovers an object from a 'change name' operation
  * returns True if anything has been recovered
@@ -277,13 +275,148 @@ static Boolean UndoChangeName(UndoListTypePtr Entry)
 	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
 	if (type != NO_TYPE)
 	{
-		SaveFree(ChangeObjectName(type, ptr1, ptr2, ptr3,
-			Entry->Data.ChangeName.Name));
+		Entry->Data.ChangeName.Name = 
+			(ChangeObjectName(type, ptr1, ptr2, ptr3,
+				Entry->Data.ChangeName.Name));
 		return(True);
 	}
 	return(False);
 }
 
+/* ---------------------------------------------------------------------------
+ * recovers an object from a 2ndSize change operation
+ */
+static Boolean UndoChange2ndSize(UndoListTypePtr Entry)
+{
+	void	*ptr1, *ptr2, *ptr3;
+	int	type;
+	Dimension	swap;
+
+		/* lookup entry by ID */
+	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
+	switch (type)
+	{
+		case VIA_TYPE:
+			swap = ((PinTypePtr) ptr2)->DrillingHole;
+			EraseVia((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->DrillingHole = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawVia((PinTypePtr) ptr2, 0);
+			break;
+		case PIN_TYPE:
+			swap = ((PinTypePtr) ptr2)->DrillingHole;
+			ErasePin((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->DrillingHole = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawPin((PinTypePtr) ptr2, 0);
+			break;
+	}
+	return(type != NO_TYPE);
+}
+
+/* ---------------------------------------------------------------------------
+ * recovers an object from a Size change operation
+ */
+static Boolean UndoChangeSize(UndoListTypePtr Entry)
+{
+	void	*ptr1, *ptr2, *ptr3;
+	int	type;
+	Dimension	swap;
+
+		/* lookup entry by ID */
+	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
+	switch (type)
+	{
+		case VIA_TYPE:
+			swap = ((PinTypePtr) ptr2)->Thickness;
+			EraseVia((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawVia((PinTypePtr) ptr2, 0);
+			break;
+		case PIN_TYPE:
+			swap = ((PinTypePtr) ptr2)->Thickness;
+			ErasePin((PinTypePtr) ptr2);
+			((PinTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawPin((PinTypePtr) ptr2, 0);
+			break;
+		case LINE_TYPE:
+			swap = ((LineTypePtr) ptr2)->Thickness;
+			EraseLine((LineTypePtr) ptr2);
+			((LineTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawLine((LayerTypePtr) ptr1, (LineTypePtr) ptr2, 0);
+			break;
+		case ARC_TYPE:
+			swap = ((ArcTypePtr) ptr2)->Thickness;
+			EraseArc((ArcTypePtr) ptr2);
+			((ArcTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawArc((LayerTypePtr) ptr1, (ArcTypePtr) ptr2, 0);
+			break;
+		case TEXT_TYPE:
+			swap = ((TextTypePtr) ptr2)->Scale;
+			EraseText((TextTypePtr) ptr2);
+			((TextTypePtr) ptr2)->Scale = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			SetTextBoundingBox(&PCB->Font, (TextTypePtr) ptr2);
+			DrawText((LayerTypePtr) ptr1, (TextTypePtr) ptr2, 0);
+			break;
+		case ELEMENTNAME_TYPE:
+			swap = ((TextTypePtr) ptr2)->Scale;
+			EraseElementName((ElementTypePtr) ptr1);
+			((TextTypePtr) ptr2)->Scale = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			SetTextBoundingBox(&PCB->Font, (TextTypePtr) ptr2);
+			DrawElementName((ElementTypePtr) ptr1, 0);
+			break;
+		case PAD_TYPE:
+			swap = ((PadTypePtr) ptr2)->Thickness;
+			ErasePad((PadTypePtr) ptr2);
+			((PadTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawPad((PadTypePtr) ptr2, 0);
+			break;
+		case ELEMENTLINE_TYPE:
+			swap = ((LineTypePtr) ptr2)->Thickness;
+			EraseElement((ElementTypePtr) ptr1);
+			((LineTypePtr) ptr2)->Thickness = Entry->Data.Size;
+			Entry->Data.Size = swap;
+			DrawElement((ElementTypePtr) ptr1, 0);
+			break;
+	}
+	return(type != NO_TYPE);
+}
+
+/* ---------------------------------------------------------------------------
+ * recovers an object from a FLAG change operation
+ */
+static Boolean UndoFlag(UndoListTypePtr Entry)
+{
+	void	*ptr1, *ptr2, *ptr3;
+	int	type, swap, tests;
+
+
+		/* lookup entry by ID */
+	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
+	if (type != NO_TYPE)
+	{
+		swap = tests = ((PinTypePtr) ptr2)->Flags;
+		tests ^= Entry->Data.Flags;
+		if (tests & DRAW_FLAGS & ~SELECTEDFLAG)
+			EraseObject(type, ptr2);
+		((PinTypePtr) ptr2)->Flags = Entry->Data.Flags;
+		Entry->Data.Flags = swap;
+		if (tests & DRAW_FLAGS)
+			DrawObject(type, ptr1, ptr2, 0);
+		return(True);
+	}
+	Message("hace Internal error: Can't find ID %d\n", Entry->ID);
+	Message("for UndoFlag Operation. Previous flags: %08x\n", Entry->Data.Flags);
+	return(False);
+}
+
 /* ---------------------------------------------------------------------------
  * recovers an object from a 'copy' or 'create' operation
  * returns True if anything has been recovered
@@ -297,7 +430,12 @@ static Boolean UndoCopyOrCreate(UndoListTypePtr Entry)
 	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
 	if (type != NO_TYPE)
 	{
-		DestroyObject(type, ptr1, ptr2, ptr3);
+		if (!RemoveList)
+			RemoveList = CreateNewBuffer();
+		EraseObject(type, ptr2);
+			/* in order to make this re-doable we move it to the RemoveList */
+		MoveObjectToBuffer(RemoveList, PCB->Data, type, ptr1, ptr2, ptr3);
+		Entry->Type = UNDO_REMOVE;	
 		return(True);
 	}
 	return(False);
@@ -318,6 +456,8 @@ static Boolean UndoMove(UndoListTypePtr Entry)
 	{
 		MoveObject(type, ptr1, ptr2, ptr3,
 			-Entry->Data.Move.DX, -Entry->Data.Move.DY);
+		Entry->Data.Move.DX *= -1;
+		Entry->Data.Move.DY *= -1;
 		return(True);
 	}
 	return(False);
@@ -338,6 +478,7 @@ static Boolean UndoRemove(UndoListTypePtr Entry)
 	{
 		DrawRecoveredObject(type, ptr1, ptr2, ptr3);
 		MoveObjectToBuffer(PCB->Data, RemoveList, type, ptr1, ptr2, ptr3);
+		Entry->Type = UNDO_CREATE;
 		return(True);
 	}
 	return(False);
@@ -350,14 +491,16 @@ static Boolean UndoRemove(UndoListTypePtr Entry)
 static Boolean UndoMoveToLayer(UndoListTypePtr Entry)
 {
 	void	*ptr1, *ptr2, *ptr3;
-	int		type;
+	int		type, swap;
 
 		/* lookup entry by it's ID */
 	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
 	if (type != NO_TYPE)
 	{
+		swap =	GetLayerNumber(PCB->Data, (LayerTypePtr) ptr1);
 		MoveObjectToLayer(type, ptr1, ptr2, ptr3, 
-			&PCB->Data->Layer[Entry->Data.MoveToLayer.OriginalLayer]);
+			&PCB->Data->Layer[Entry->Data.MoveToLayer.OriginalLayer], True);
+		Entry->Data.MoveToLayer.OriginalLayer = swap;
 		return(True);
 	}
 	return(False);
@@ -385,11 +528,13 @@ static Boolean UndoRemovePoint(UndoListTypePtr Entry)
 			if (layer->On)
 				ErasePolygon(polygon);
 			InsertPointIntoObject(POLYGON_TYPE, layer, polygon,
-				&polygon->Points[Entry->Data.RemovedPoint.Index],
+				&Entry->Data.RemovedPoint.Index,
 				Entry->Data.RemovedPoint.Point.X,
 				Entry->Data.RemovedPoint.Point.Y);
 			if (layer->On)
-				DrawPolygon(layer, polygon);
+				DrawPolygon(layer, polygon, 0);
+			Entry->Type = UNDO_INSERT_POINT;
+			Entry->ID = polygon->Points[Entry->Data.RemovedPoint.Index].ID;
 			return(True);
 		}
 
@@ -406,6 +551,7 @@ static Boolean UndoInsertPoint(UndoListTypePtr Entry)
 {
 	void	*ptr1, *ptr2, *ptr3;
 	int		type;
+	PointTypePtr	pnt;
 
 		/* lookup entry by it's ID */
 	type = SearchObjectByID(PCB->Data, &ptr1, &ptr2, &ptr3, Entry->ID);
@@ -413,8 +559,19 @@ static Boolean UndoInsertPoint(UndoListTypePtr Entry)
 	{
 		case POLYGONPOINT_TYPE:		/* removes an inserted polygon point */
 		{
-			DestroyObject(POLYGONPOINT_TYPE, (LayerTypePtr) ptr1,
+			pnt = (PointTypePtr) ptr3;
+			Entry->Data.RemovedPoint.Point.X = pnt->X;
+			Entry->Data.RemovedPoint.Point.Y = pnt->Y;
+			Entry->ID = ((PolygonTypePtr) ptr2)->ID;
+			POLYGONPOINT_LOOP((PolygonTypePtr) ptr2,
+				if (pnt == point)
+				{
+					Entry->Data.RemovedPoint.Index = n;
+					break;
+				});
+			DestroyObject(PCB->Data, POLYGONPOINT_TYPE, (LayerTypePtr) ptr1,
 				(PolygonTypePtr) ptr2, (PointTypePtr) ptr3);
+			Entry->Type = UNDO_REMOVE_POINT;
 			return(True);
 		}
 
@@ -425,7 +582,6 @@ static Boolean UndoInsertPoint(UndoListTypePtr Entry)
 
 /* ---------------------------------------------------------------------------
  * undo of any 'hard to recover' operation
- * supported are MOVE, CREATE, REMOVE and CHANGENAME
  *
  * returns True if anything is has been done
  */
@@ -435,7 +591,10 @@ Boolean Undo(void)
 	Boolean			changed = False;
 
 	if (!UndoN)
+	{
+		Message("Nothing to undo - buffer is empty\n");
 		return(False);
+	}
 
 		/* lock undo module to prevent from loops
 		 * and loop over all entries with the same serial number
@@ -443,7 +602,7 @@ Boolean Undo(void)
 	LockUndo();
 	ptr = &UndoList[UndoN -1];
 	Serial = ptr->Serial;
-	for (; UndoN && ptr->Serial == Serial; ptr--, UndoN--)
+	for (; UndoN && ptr->Serial == Serial; ptr--, UndoN--, RedoN++)
 		switch(ptr->Type)
 		{
 			case UNDO_CHANGENAME:
@@ -477,10 +636,109 @@ Boolean Undo(void)
 
 			case UNDO_MOVETOLAYER:
 				changed |= UndoMoveToLayer(ptr);
-		}
+				break;
 
+			case UNDO_FLAG:
+				changed |= UndoFlag(ptr);
+				break;
+
+			case UNDO_CHANGESIZE:
+				changed |= UndoChangeSize(ptr);
+				break;
+
+			case UNDO_CHANGE2NDSIZE:
+				changed |= UndoChange2ndSize(ptr);
+				break;
+
+			case UNDO_BLOCK:
+				UndoBlock(ptr);
+		}
 		/* release lock */
 	UnlockUndo();
+	if (changed)
+	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
+	return(changed);
+}
+
+/* ---------------------------------------------------------------------------
+ * redo of any 'hard to recover' operation
+ *
+ * returns True if anything is has been done
+ */
+Boolean Redo(void)
+{
+	UndoListTypePtr	ptr;
+	Boolean			changed = False;
+
+	if (!RedoN)
+	{
+		Message("Nothing to redo. Perhaps changes have been made since last undo\n");
+		return(False);
+	}
+
+		/* lock undo module to prevent from loops
+		 * and loop over all entries with the same serial number
+		 */
+	LockUndo();
+	ptr = &UndoList[UndoN];
+	Serial = ptr->Serial;
+	for (; RedoN && ptr->Serial == Serial; ptr++, UndoN++, RedoN--)
+		switch(ptr->Type)
+		{
+			case UNDO_CHANGENAME:
+				changed |= UndoChangeName(ptr);
+				break;
+
+			case UNDO_CREATE:
+			case UNDO_COPY:
+				changed |= UndoCopyOrCreate(ptr);
+				break;
+
+			case UNDO_MOVE:
+				changed |= UndoMove(ptr);
+				break;
+
+			case UNDO_REMOVE:
+				changed |= UndoRemove(ptr);
+				break;
+
+			case UNDO_REMOVE_POINT:
+				changed |= UndoRemovePoint(ptr);
+				break;
+
+			case UNDO_INSERT_POINT:
+				changed |= UndoInsertPoint(ptr);
+				break;
+
+			case UNDO_ROTATE:
+				changed |= UndoRotate(ptr);
+				break;
+
+			case UNDO_MOVETOLAYER:
+				changed |= UndoMoveToLayer(ptr);
+				break;
+
+			case UNDO_FLAG:
+				changed |= UndoFlag(ptr);
+				break;
+
+			case UNDO_CHANGESIZE:
+				changed |= UndoChangeSize(ptr);
+				break;
+
+			case UNDO_CHANGE2NDSIZE:
+				changed |= UndoChange2ndSize(ptr);
+				break;
+
+			case UNDO_BLOCK:
+				UndoBlock(ptr);
+				break;
+		}
+	/* Make next serial number current in case we take a new branch */
+	Serial++;
+	UnlockUndo();
+	if (changed)
+	        XSendEvent(Dpy, Output.OutputWindow, False, ExposureMask, (XEvent *) &Erasure);
 	return(changed);
 }
 
@@ -489,7 +747,7 @@ Boolean Undo(void)
  */
 void RestoreUndoSerialNumber(void)
 {
-	Serial = SavedSerial;
+		Serial = SavedSerial;
 }
 
 /* ---------------------------------------------------------------------------
@@ -522,19 +780,38 @@ void ClearUndoList(Boolean Force)
 	{
 			/* release memory allocated by objects in undo list */
 		for (undo = UndoList; UndoN; undo++, UndoN--)
+		{
 			if (undo->Type == UNDO_CHANGENAME)
 				SaveFree(undo->Data.ChangeName.Name);
+			if (undo->Type == UNDO_BLOCK)
+				SaveFree(undo->Data.Block.Box);
+		}
 		MyFree((char **) &UndoList);
 		FreeDataMemory(RemoveList);
 
 			/* reset some counters */
-		UndoN = UndoMax = 0;
+		UndoN = UndoMax = RedoN = 0;
 	}
 
 		/* reset counter in any case */
 	Serial = 0;
 }
 
+/* ---------------------------------------------------------------------------
+ * marks the block and box boundary for block drawing during undo
+ */
+void AddBlockToUndoList(BoxTypePtr Box)
+{
+	UndoListTypePtr undo;
+
+	if (!Locked)
+	{
+		undo = GetUndoSlot(UNDO_BLOCK, 0);
+		undo->Data.Block.Box = NULL;
+		if (Box && (undo->Data.Block.Box = malloc(sizeof(BoxType))))
+			*undo->Data.Block.Box = *Box;
+	}
+}
 /* ---------------------------------------------------------------------------
  * adds an object to the list of rotated objects
  */
@@ -574,34 +851,23 @@ void MoveObjectToRemoveUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
  * adds an object to the list of removed polygon/... points
  */
 void AddObjectToRemovePointUndoList(int Type,
-	void *Ptr1, void *Ptr2, void *Ptr3)
+	void *Ptr1, void *Ptr2, Cardinal index)
 {
 	UndoListTypePtr	undo;
 	PolygonTypePtr	polygon = (PolygonTypePtr) Ptr2;
-	PointTypePtr	pnt = (PointTypePtr) Ptr3;
 
 	if (!Locked)
 	{
-			/* save the ID of the parent object; else it will be
-			 * impossible to recover the point
-			 */
-		undo = GetUndoSlot(UNDO_REMOVE_POINT, OBJECT_ID(polygon));
-		undo->Data.RemovedPoint.Point = *pnt;
-
 		switch(Type)
 		{
 			case POLYGONPOINT_TYPE:
 			{
-					/* get index of previous point in array */
-				POLYGONPOINT_LOOP(polygon,
-					if (point == pnt)
-					{
-							/* use n-2 because the point isn't removed yet */
-						undo->Data.RemovedPoint.Index = (n ?
-							n-1 : polygon->PointN-2);
-						break;
-					}
-				);
+				/* save the ID of the parent object; else it will be
+				 * impossible to recover the point
+				 */
+				undo = GetUndoSlot(UNDO_REMOVE_POINT, OBJECT_ID(polygon));
+				undo->Data.RemovedPoint.Point = polygon->Points[index];
+				undo->Data.RemovedPoint.Index = index;
 			}
 			break;
 		}
@@ -663,7 +929,7 @@ void AddObjectToChangeNameUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3,
 }
 
 /* ---------------------------------------------------------------------------
- * adds an object to the list of objects with changed names
+ * adds an object to the list of objects moved to another layer
  */
 void AddObjectToMoveToLayerUndoList(int Type, void *Ptr1, void *Ptr2,
 	void *Ptr3)
@@ -689,6 +955,85 @@ void AddObjectToCreateUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
 		undo = GetUndoSlot(UNDO_CREATE, OBJECT_ID(Ptr3));
 }
 
+/* ---------------------------------------------------------------------------
+ * adds an object to the list of objects with flags changed
+ */
+void AddObjectToFlagUndoList(int Type, void *Ptr1, void *Ptr2, void *Ptr3)
+{
+	UndoListTypePtr	undo;
+
+	if (!Locked) {
+		undo = GetUndoSlot(UNDO_FLAG, OBJECT_ID(Ptr2));
+		undo->Data.Flags = ((PinTypePtr) Ptr2)->Flags;
+	}
+}
+
+/* ---------------------------------------------------------------------------
+ * adds an object to the list of objects with Size changes
+ */
+void AddObjectToSizeUndoList(int Type, void *ptr1, void *ptr2, void *ptr3)
+{
+	UndoListTypePtr undo;
+
+	if (!Locked)
+	{
+		undo = GetUndoSlot(UNDO_CHANGESIZE, OBJECT_ID(ptr2));
+		switch (Type)
+		{
+			case PIN_TYPE:
+			case VIA_TYPE:
+				undo->Data.Size = ((PinTypePtr) ptr2)->Thickness;
+				break;
+			case LINE_TYPE:
+			case ELEMENTLINE_TYPE:
+				undo->Data.Size = ((LineTypePtr) ptr2)->Thickness;
+				break;
+			case TEXT_TYPE:
+			case ELEMENTNAME_TYPE:
+				undo->Data.Size = ((TextTypePtr) ptr2)->Scale;
+				break;
+			case PAD_TYPE:
+				undo->Data.Size = ((PadTypePtr) ptr2)->Thickness;
+				break;
+			case ARC_TYPE:
+			case ELEMENTARC_TYPE:
+				undo->Data.Size = ((ArcTypePtr) ptr2)->Thickness;
+				break;
+		}
+	}
+}
+
+/* ---------------------------------------------------------------------------
+ * adds an object to the list of objects with 2ndSize changes
+ */
+void AddObjectTo2ndSizeUndoList(int Type, void *ptr1, void *ptr2, void *ptr3)
+{
+	UndoListTypePtr undo;
+
+	if (!Locked)
+	{
+		undo = GetUndoSlot(UNDO_CHANGE2NDSIZE, OBJECT_ID(ptr2));
+		if (Type == PIN_TYPE || Type == VIA_TYPE)
+			undo->Data.Size = ((PinTypePtr) ptr2)->DrillingHole;
+	}
+}
+
+/* ---------------------------------------------------------------------------
+ * aborts a false block start
+ */
+void UndoFalseBlock(void)
+{
+	UndoListTypePtr undo = &UndoList[UndoN - 1];
+
+	if (!UndoN || undo->Type != UNDO_BLOCK)
+		Message("hace: Internal error in UndoFalseBlock\n");
+	else
+	{
+		SaveFree(undo->Data.Block.Box);
+		UndoN--;
+	}
+}
+
 /* ---------------------------------------------------------------------------
  * set lock flag
  */
